
/*******************************************************************************************
 *                AVR Touch Screen (ET-TFT240320TP-3.2 Rev.C) Treiber     		           *
 *                                                                                         *
 *                            DISPLAY PORTRAIT(Vertikal)                                   *
 *                                                                                         *
 *                   Interface :GLCD Data 16bit MODE (USE Device ID=0(IM0=0))              *
 *******************************************************************************************
 *                                                                                         *
 * Target MCU     : ATMEL AVR-MEGA2560                                                     *
 *       	      : X-TAL : 16 MHz                                                         * 
 * Editor         : AVR Studio 4.15 Build 623                                              *
 * Compiler       : WinAVR-20081205                                                        *
 * Create By      : Mr. Sittiphol Yooyod (WWW.ETT.CO.TH)                                   *
 * Last Update    : 25/April/2011                                                          *
 *																				           *
 * Port Interface :  -- MCU-Mega2560--	  	  	      --LCD-ET-TFT240320TP-3.2 Rev.C--     *
 *                                                                                         *
 *     DAta GLCD  :     PA[0..7]          Connect          DB0..DB7                        * 
 *                      PC[0..7]          Connect          DB8..DB15                       *
 *                      				                                                   *
 *     Contrl GLCD:     PE2               Connect          CS-H						       *
 *                      PE3               Connect          RS-H(0=cmm,1=data)              *
 *                      PE4               Connect          WR-H(Write)                     *
 *                      PE5               Connect          RD-H(Read)                      *
 *                      PE6               Connect          RES-H(Reset)                    *
 *     Back Light :     PE7               Connect          BL-H(LCD-Back Light(Active=1))  *
 *   																				       *
 *     Touch Sceen:    PH2 (MISO-in)	  Connect 		    MISO					       *
 *                     PH3 (MOSI-out)	  Connect 		    MOSI					       *
 *                     PH4 (SCLK-out)     Connect 		    SCLK					       *
 *                     PH5 (BUSY-in)	  Connect 		    BUSY(No_USE)				   *
 *                     PH6 (PEN-in)	      Connect 		    PEN					           *
 *                                                                                         *
 *******************************************************************************************/


// interne Headerdateien sind standardmssig im AVR-Studio verfgbar
#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>                  //define port
#include <compat/deprecated.h>       //call function sbi(port,bit),cbi(port,bit) = Set,Clear bit
#include <avr/pgmspace.h>            //for progrm data to Area Flash
#include <avr/wdt.h>                 // Watchdog
#include <util/delay.h>              //call function delay

// zustzliche Headerdateien mssen im Projekt Verzeichniss vorhanden sein oder angelegt werden
#define extern
#include "ett240320tp32font.h"
#include "ett240320tp32revb.h"
#undef extern





//********** Valiable for Touch Screen Function GLCD ************
long dis_XD[3] = {24,215,120}          ;    // Value refer Point X at 10% of Display(X=240:0-239) 3 Position
long dis_YD[3] = {32,160,287}          ;    // Value refer Point Y at 10% of Display(Y=320:0-319) 3 Position


/*****************************************
 **         Function Delay ms           **
 *****************************************/
 void delay_ms(unsigned int cnt)
  {
    for(;cnt>0;cnt--)
       _delay_ms(1);
  }

/*##########**********************************************************************************#########
  ##########**                                                                              **#########
  ##########** 1.Function Control of Graphic LCD EL-TFT240320TP-3.2 REV.B(Driver SPFD5408S) **#########
  ##########**                                                                              **#########                 
  ##########**********************************************************************************######### */

//******** Control Pin cs(PE2=CS) ********
void gp_cs_hi(void) {sbi(PORTE,2);}	 // PE2 = Hi
void gp_cs_lo(void) {cbi(PORTE,2);}	 // PE2 = Lo

//******* Control Pin rs (PE3=RS) *******
void gp_rs_hi(void) {sbi(PORTE,3);}	 // PE3 = Hi
void gp_rs_lo(void) {cbi(PORTE,3);}  // PE3 = Lo

//******* Control Pin wr(PE4=WR) *********
void gp_wr_hi(void) {sbi(PORTE,4);}  // PE4 = Hi
void gp_wr_lo(void) {cbi(PORTE,4);}  // PE4 = Lo

//****** Control Pin rd(PE5=RD) ******
void gp_rd_hi(void) {sbi(PORTE,5);}  // PE5 = Hi
void gp_rd_lo(void) {cbi(PORTE,5);}  // PE5 = Lo

//******** Control Pin res(PE6=RES) *********
void gp_res_hi(void){sbi(PORTE,6);}  // PE6 = Hi
void gp_res_lo(void){cbi(PORTE,6);}  // PE6 = Lo

//********* Control Back Light(PE7=Back Light)**********
void gp_bl_on(void) {sbi(PORTE,7);}  // PE7 = Hi(ON)
void gp_bl_off(void){cbi(PORTE,7);}  // PE7 = Lo(OFF)



//*********************************************************
//************* Write Address Command to LCD **************
//*********************************************************

void gp_wr_cmm(unsigned char cmm)
 {
    gp_cs_lo()                        ;
    //-----Write the command to the data bus.----
    PORTA  = cmm                      ;
    PORTC  = 0x00                     ;
    //----Set the RS signal low, indicating a command.-----
    gp_rs_lo()                        ;  //RS=0
    //-----Assert the write enable signal.-------
	gp_wr_lo()                        ;  //wr=0 
    //------Deassert the write enable signal.-------
   	gp_wr_hi()	                       ;  //wr=1
    //------Set the RS signal high, indicating that following writes are data.------
    gp_rs_hi()                        ;  //RS=1
 }


//***********************************************
//************* Write data to LCD ***************
//***********************************************

void gp_wr_data(unsigned int dw)
 {	
    gp_cs_lo()                           ;
    //----- Write data to data bus. 16 bit PA[0..7],PC[8..15] ------
	PORTA    = dw                        ;
    PORTC    = dw>>8                     ;
	//----- Assert the write enable signal.--------
    gp_wr_lo()                           ;  //wr = 0
    //------ Deassert the write enable signal.-------- 
    gp_wr_hi()                           ;  //wr = 1
    gp_cs_hi()                           ;
 }


//***************************************************** 
//***************** Initial GLCD **********************
//*****************************************************

void init_glcd(void)
 {
   DDRA  = 0xFF     ;        //Set PortA(0..7) = output for data GLCD Byte Low
   DDRC  = 0xFF     ;        //Set PortC(0..7) = Output for data GLCD Byte Hig
   DDRE |= 0xFC     ;        //Set PortE(2..7) = Output for Control GLCD(CS,RS,WR,RD,Reset,BL)  
   DDRH  = 0x98     ;        //Set PortH = PH7,PH4,PH3:Output ;PH6,PH5,PH2:Input for Touch Screen
   
   //------ Set Value Default Port ------ 
   PORTA = 0        ;       
   PORTC = 0        ;
   PORTE = 0x7C     ;
   PORTH = 0x80     ;
   delay_ms(10)     ;

   long cnt ;
    
    //-------------- Start GLCD --------------

	  gp_rd_hi()            ; 

  	  gp_res_lo()           ;    // Reset GLCD 
      delay_ms(1)           ;    // Delay for 1ms.	 
      gp_res_hi()           ;    // Stop Signal Reset GLCD    
      delay_ms(1)           ;	 // Delay for 1ms while the LCD comes out of reset.
   
  
      gp_wr_cmm(0x00)       ;	 // Command Start Oscillater
      gp_wr_data(0x0001)    ;	 // Enable the oscillator.
   
      delay_ms(10)          ;    // Delay for 10ms while the oscillator stabilizes.
   

	 //----- Display Setting -----
	 	
	  gp_wr_cmm(0x01)       ;	 // Configure the output drivers.
      gp_wr_data(0x0100)    ;
      gp_wr_cmm(0x02)       ;	 // Configure the LCD A/C drive waveform.
      gp_wr_data(0x0700)    ;	 // Line inversion 0x0700
      gp_wr_cmm(0x03)       ;    // Configure Entry Mode
      gp_wr_data(0x1230)    ; 	 // Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.
   

   //------ Configure and enable the LCD power supply.-------

	 //---- Power Control -----

	  gp_wr_cmm(0x07)       ;    // Enable internal operation of the LCD controller.
      gp_wr_data(0x0101)    ;	 // 0011
	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x0000)    ;  //0x00B0
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0007)    ;   //0x0037
	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x0000)    ;  //0x011E
	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x0000)    ;  //0x1A00
	  delay_ms(20)          ;

	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x16B0)    ;
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0037)    ;
	  delay_ms(20)          ; //50

	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x013E)    ;
	  delay_ms(20)          ;  //50

	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x1A00)    ;
      gp_wr_cmm(0x29)       ;
      gp_wr_data(0x000F)    ;      
      delay_ms(20)          ;    //50- Delay for 40ms while the power supply stabilizes.
    
   
	 
	  //------- Clear the contents of the display buffer. --------
   
      gp_wr_cmm(0x22);


      for(cnt = 0;cnt<((long)240*320);cnt++)
         gp_wr_data(Schwarz)   ;    // Back-Ground Color Black
      
   

	  gp_wr_cmm(0x20)       ;    //Command Horizontal GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 
	  gp_wr_cmm(0x21)       ;    //Command Vertical GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 

	  gp_wr_cmm(0x50)       ;  	 //Command Horizontal Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x51)       ;  	 //Command Horizontal Address End Position(239)
	  gp_wr_data(0x00EF)    ;  

	  
	  gp_wr_cmm(0x52)       ;  	 //Command Vertical Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x53)       ;  	 //Command Verticall Address End Position(239)
	  gp_wr_data(0x013F)    ;  
  

      gp_wr_cmm(0x60)       ;    // Set the number of lines to scan.
      gp_wr_data(0xA700)    ;    // GS=1                                                    ############## Edit ###############

      gp_wr_cmm(0x61)       ;    // Enable grayscale inversion of the source outputs.
      gp_wr_data(0x0001)    ;

    
   //------- GAMMA Control -------

	  gp_wr_cmm(0x30)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x31)       ; 
	  gp_wr_data(0x0403)    ;
  	  gp_wr_cmm(0x32)       ; 
	  gp_wr_data(0x0404)    ;
      gp_wr_cmm(0x35)       ; 
	  gp_wr_data(0x0002)    ;
      gp_wr_cmm(0x36)       ; 
	  gp_wr_data(0x0707)    ;
	  gp_wr_cmm(0x37)       ; 
	  gp_wr_data(0x0606)    ;
      gp_wr_cmm(0x38)       ; 
	  gp_wr_data(0x0106)    ; 
	  gp_wr_cmm(0x39)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x3C)       ; 
	  gp_wr_data(0x0700)    ;
  	  gp_wr_cmm(0x3D)       ; 
	  gp_wr_data(0x0707)    ;

    
      gp_wr_cmm(0x07)       ;	 //Enable the image display.
      gp_wr_data(0x0173)    ;	 
	
      gp_bl_on()            ;	 //Back Light ON
	  
	  delay_ms(20)          ;	 //Delay for 20ms, which is equivalent to two frames.

 }


/* #########*********************************************************#############
   #########**                                                     **############# 
   #########**   2.Function Touch Screen (ADS7846:SPI-Interface)   **#############
   #########**                                                     **#############            
   #########*********************************************************#############	*/

//*********** CS [PH7] *************
 void tcs_cs_hi(void) {sbi(PORTH,7);}   // PH7 =1
 void tcs_cs_lo(void) {cbi(PORTH,7);}   // PH7 =0

//*********** SCK [PH4] **************
 void tcs_sck_hi(void){sbi(PORTH,4);}   // PH4 =1
 void tcs_sck_lo(void){cbi(PORTH,4);}   // PH4 =0

//*********** Data MOSI [PH3]************
 void tcs_sdo_hi(void){sbi(PORTH,3);}   // PH3 =1
 void tcs_sdo_lo(void){cbi(PORTH,3);}   // PH3 =0



///************************************************************************
//*******          Function Check Status Press Touch Screen       ********
//************************************************************************
//******** Return Value : 											   ***
//********               Function return value '0'= Not Touched screen ***
//********               Function return value '1'= Touched screen     ***
//************************************************************************


int8_t status_touch(void)
 {
  
  if((PINH & 0x40)== 0x40)	//Read PH6
	return 0 ;			    //No Touch
  else
    return 1 ;			    //Press Touch screen
 }


 //****************************************************************************************
 //******      Function SPI Write and Read Data 1 Byte from Touch Screen ADS7846     ******
 //****************************************************************************************
 //******       Parameter : 												         ******
 //******                  DataByte = data or command control ADS7846 (1Byte)        ****** 
 //******    Return Value : 														 ******
 //******                  Return value adc from touched times 1 byte Pass Function  ******         
 //****************************************************************************************

unsigned char tcs_wr(unsigned char DataByte)			  	
{
  unsigned char Bit,result=0 ;				 	   

     for(Bit=0;Bit<8;Bit++)                  // 8 Bit Write
      {		 
	   	tcs_sck_hi()                     ;	 // Start Clock  High

		//--------- write Data(PH3) ---------

		if((DataByte & 0x80)== 0x80)
		   tcs_sdo_hi()                  ;   // Set bit data(PD3) = 1
		else
		   tcs_sdo_lo()                  ;	 // Clear bit data(PD3) = 0
		   	
		tcs_sck_lo()                     ;   // Clock low

		DataByte <<= 1                   ;	 // Shift Next Bit Data
      
        
	  	//---------- Read Data(PH2) -------

	    result <<= 1                   ;	
		 										  
	     if ((PINH & 0x04)== 0x04)                // If SDI = "1"						
	      {
	       result |= 0x01                ;       // Keep Result of Read-data  									
	      }
			  
	  }

	  return (result)                    ;
}




//******************************************************************************** 
//******    Function Read X-Y-Position ADC Touch Screen-12 bit adc          ******
//********************************************************************************
//******        Parameter :	No                                              ******
//******    Return Value  :													******
//******                    dif_adc_X = Keep Result ADC X-Position(12 bit)  ******
//******                    dif_adc_Y = Keep result ADC Y-Position(12 bit)  ******                      
//********************************************************************************

void tcs_read(void)
 {
    unsigned int tcs_adc_X ,tcs_adc_Y       ;
	unsigned char buf_data[4]               ;

	if(status_touch())
	{
	//--- Set default Pin control Touch Screen ----
	  tcs_sdo_lo() ;
	  tcs_sck_lo() ;
	  tcs_cs_lo () ;


      buf_data[0] = tcs_wr(0xD0)            ;	//Write Command Measure X-Position 
	  buf_data[0] = tcs_wr(0x00)            ;	//Read ADC data X-Position (7-bit byte High) data: 0ddddddd	(bit) 
	  buf_data[1] = tcs_wr(0x90)            ;	//Write Command Measure Y-Position ; Read ADC data X-Position (5-bit byte Low)  data:ddddd000(bit)
	  buf_data[2] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(7-bit byte High) data: 0ddddddd (bit)
	  buf_data[3] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(5-bit byte Low)  data: ddddd000 (bit)

	  tcs_adc_X  = buf_data[0]              ;	// Mark ADC Data X-Position 12 bit
	  tcs_adc_X  = tcs_adc_X << 5           ;
	  tcs_adc_X |= buf_data[1] >> 3         ;
	  tcs_adc_X  = tcs_adc_X & 0x0FFF       ;
	 
	  tcs_adc_Y  = buf_data[2]              ;   // Mark ADC Data Y-Position 12 bit
	  tcs_adc_Y  = tcs_adc_Y << 5           ;   // Shift 7 bit High
	  tcs_adc_Y |= buf_data[3] >> 3         ;   // Shift 5 bit low
	  tcs_adc_Y  = tcs_adc_Y & 0x0FFF       ;   // total ADC data 12 bit
	

	  tcs_cs_hi()                           ;
	 
	 
	  dif_adc_X = tcs_adc_X                 ;	//ADC 12 bit :LCD Start Portrait
	  dif_adc_Y = 4095-tcs_adc_Y	        ;

   }
 }



//*********************************************************************************************
//******  Function Calculate average of measurements,by cut value max and min measurement. ****
//*********************************************************************************************
//******      Parameter :					                                          *********
//******			     num = number times of measurement adc from touch screen      *********
//******    Return value:														      *********
//******                 ave_adc_X = Keep result average adc X-Position			      *********
//******                 ave_adc_Y = Keep result average adc Y-Position               *********     
//*********************************************************************************************

void ave_x_y(unsigned char num)				
{
	unsigned char i;
	long sum_x, sum_y;
	long min_x, min_y, max_x, max_y;

	sum_x=0;sum_y=0;min_x=0xffff;min_y=0xffff,max_x=0;max_y=0;

	for(i=0;i<num;i++)
	{
		sum_x += buf_adc_X[i]         ;
		sum_y += buf_adc_Y[i]         ;

		if(buf_adc_X[i]< min_x)		                         //search for minimum X value
		{
			min_x = buf_adc_X[i]      ;
		}
		else if (buf_adc_X[i]> max_x)	                    //search for maximum X value
		{
			max_x = buf_adc_X[i]      ;
		}

		if(buf_adc_Y[i]< min_y)	  	                        //search for minimum Y value
		{
			min_y = buf_adc_Y[i]      ;
		}
		else if(buf_adc_Y[i]> max_y)	  	                //search for maximum Y value
		{
			max_y = buf_adc_Y[i]      ;
		}
	}
	ave_adc_X = (sum_x - min_x - max_x ) / (num-2) ;  	   //update x average value
	ave_adc_Y = (sum_y - min_y - max_y ) / (num-2) ;	   //update y average value
}




//****************************************************************************
//********* Function Calculate Matrix Find value Coeffcient            *******
//****************************************************************************
//*********     Parameter  : No                                        *******
//*********   Return Value : Keep value Coeffcient for use calculate   *******                                      ***********
//*********                  X-Y-Position in variable divider,An,Bn,Cn *******                   
//*********                  Dn,En,Fn                                  *******
//****************************************************************************

 void set_matrix()
  {
	divider =  	((tcs_ave_X[0]-tcs_ave_X[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
	            ((tcs_ave_X[1]-tcs_ave_X[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))      ;


         An = 	((dis_XD[0]-dis_XD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_XD[1]-dis_XD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

	 	 Bn = 	((tcs_ave_X[0]-tcs_ave_X[2])*(dis_XD[1]-dis_XD[2]))-
				((dis_XD[0]-dis_XD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Cn =  	(tcs_ave_X[2]*dis_XD[1]-tcs_ave_X[1]*dis_XD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_XD[2]-tcs_ave_X[2]*dis_XD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_XD[0]-tcs_ave_X[0]*dis_XD[1])*tcs_ave_Y[2]   ;

		 Dn =   ((dis_YD[0]-dis_YD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_YD[1]-dis_YD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

		 En =   ((tcs_ave_X[0]-tcs_ave_X[2])*(dis_YD[1]-dis_YD[2]))-
				((dis_YD[0]-dis_YD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Fn =  	(tcs_ave_X[2]*dis_YD[1]-tcs_ave_X[1]*dis_YD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_YD[2]-tcs_ave_X[2]*dis_YD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_YD[0]-tcs_ave_X[0]*dis_YD[1])*tcs_ave_Y[2]   ;

  }


//****************************************************************************
//*********** Function Get Position Address real of Display ******************
//****************************************************************************
//***         Parameter  :   										      ****
//***                     num = Number times for Measure adc from Touched **** 
//***       Return Value :											      ****
//***                       X = Keep value address X-Position		      ****
//***                       Y = Keep value address Y-Position             **** 
//****************************************************************************

void get_point(unsigned char num)
 {
    unsigned char nm  ; 
	
		  nm=0                              ;
		  while(status_touch() && nm<num)	  //Measure touch x,y 10 times if the Touch kept pressed,
		   {
			  tcs_read()                    ; //Read value ADC Touch X-Y 
			  buf_adc_X[nm]=	dif_adc_X   ; //keep value ADC Touch-X
		      buf_adc_Y[nm]=	dif_adc_Y   ; //keep value ADC Touch-Y

 			  nm++                          ; 
		   }

		  if(nm==num)	                      //if the touch xy successfuly collected,
		   {	
				ave_x_y(num)                ; //take average out of 10 touch measurements, excluding max and min measurement.
			    
				dif_adc_X = ave_adc_X       ; //keep average ADC Touch-X
				dif_adc_Y = ave_adc_Y       ; //keep average ADC Touch-Y
               
			    X = ((An*dif_adc_X)+(Bn*dif_adc_Y)+Cn)/divider  ; 
	            Y = ((Dn*dif_adc_X)+(En*dif_adc_Y)+Fn)/divider  ;
		   }

 }


/*******************************************
 Funktion Background Farbe oder Clear Screen
 *******************************************
 Parameter:
 - bg_color = Hintergrund Farbe
 *******************************************/
void bg_color(uint16_t bg_color)
  {
   long cnt             ;
   gp_wr_cmm(0x20)      ; // Command Horizontal GRAM Address Set
   gp_wr_data(0x0000)   ; //Address +Start 0x0000
   gp_wr_cmm(0x21)      ; //Command Vertical GRAM Address Set
   gp_wr_data(0x0000)   ; //Address Start 0x0000
   gp_wr_cmm(0x22)      ;
   for(cnt = 0;cnt<((long)240*320);cnt++)
   gp_wr_data(bg_color) ; // Back-Ground Color Black
  }


/******************************************************
 Function Print Text 1 Charecter
 ******************************************************
 Parameter :
 - row      = Ascii Code (Position buffer keep text)
 - adx,ady  = Position X,Y for begin plot text by will
              begin plot from bottom left to top left
 - fg_clr   = Color of text
 - bg_clr   = Color background of text
              if bg_clr =no_bg or1=non color background
 ******************************************************/
void lcd_Ascii(uint8_t row,uint8_t font, uint8_t adx,uint16_t ady,uint16_t fg_clr,uint16_t bg_clr)
 {
   uint16_t ay        ;
   uint8_t ax,m,s,n,tx;
   ax = adx           ;
   ay = ady           ;
   row = row-0x20;
   s=0; tx=0; n=0;

   // ********** Print Text 1 Charecter(data 6,14,42 Byte) **********
   switch(font)
         {
	 	   case 1: s =   6; break; // Font1 =  6 Bytes
	       case 2: s =  14; break; // Font2 = 14 Bytes
	       case 3: s =  88; break; // Font3 = 88 Bytes
		   case 4: s = 168; break; // Font4 =168 Bytes
         };

   for(m=0;m<s;m++)
	  {
	    // **** Sent data byte1=8 bit ****
		switch(font)
		      {
			    case 1: tx = pgm_read_byte(&(ascii_6x8[row][m]));   break; // Lese Ascii Zeichen Font 1 (6x8)
                case 2: tx = pgm_read_byte(&(ascii_7x11[row][m]));  break; // Lese Ascii Zeichen Font 2 (7x11)
				case 3: tx = pgm_read_byte(&(ascii_22x32[row][m])); break; // Lese Ascii Zeichen Font 3 (14x24)
				case 4: tx = pgm_read_byte(&(ascii_24x56[row][m])); break; // Lese Ascii Zeichen Font 4 (24x56)
              }
		for(n=0;n<8;n++)		      // Loop Sent data  1 byte(8bit)
		   {
	         if  (tx & 0x80)	      // if data bit7 = 1 ,Plot Color area Charecter
	 		     {
	 		       gp_wr_cmm(0x20)   ; // Command Set Adddress Hor(X)
                   gp_wr_data(ax)    ; // Sent X_Address CGRAM
                   gp_wr_cmm(0x21)   ; // Command Set Address Ver(Y)
	               gp_wr_data(ay)    ; // Sent Y_Address CGRAM
				   gp_wr_cmm(0x22)   ; // Command Write data
                   gp_wr_data(fg_clr);
 			     }
		     else{					  // if data bit7 = 0 ,Plot Color area back ground Charecter
			     
			       if  (bg_clr != 1)
                       {
			             gp_wr_cmm(0x20)   ; // Command Set Adddress Hor(X)
                         gp_wr_data(ax)    ; // Sent X_Address CGRAM
                         gp_wr_cmm(0x21)   ; // Command Set Adddress Ver(Y)
                         gp_wr_data(ay)    ; // Sent Y_Address CGRAM
			             gp_wr_cmm(0x22)   ; // Command Write data
                         gp_wr_data(bg_clr); // Sent Data
			           }
			     }
		     tx <<= 1                 ; // Shift Right data 1 bit
		     ay   = ay-1              ; // Decrement Y-address
		   }
        // **** Sent data byte2=3bit (nur Font2) ****
        if  (font == 2)
            {
			  m++                      ; // Increment Next pointter byte Data
  		      tx = pgm_read_byte(&(ascii_7x11[row][m])); // Read data byte2
 		      for(n=0;n<3;n++)	       // Loop sent data byte2 = 3 bit
		         {
	               if  (tx & 0x80)		   // if data bit7 = 1 ,Plot Color area Charecter
			           {
			             gp_wr_cmm(0x20)   ; // Command Set Adddress Hor(X)
                         gp_wr_data(ax)    ; // Sent X_Address CGRAM
                         gp_wr_cmm(0x21)   ; // Command Set Adddress Ver(Y)
                         gp_wr_data(ay)    ; // Sent Y_Address CGRAM
			             gp_wr_cmm(0x22)   ; // Command Write data
                         gp_wr_data(fg_clr);
 			           }
		           else{				      // if data bit7 = 0 ,Plot Color area back ground Charecter
			             if  (bg_clr != 1)
                             {
			                   gp_wr_cmm(0x20)   ; // Command Set Adddress Hor(X)
                               gp_wr_data(ax)    ; // Sent X_Address CGRAM
                               gp_wr_cmm(0x21)   ; // Command Set Adddress Ver(Y)
                               gp_wr_data(ay)    ; // Sent Y_Address CGRAM
			                   gp_wr_cmm(0x22)   ; // Command Write data
                               gp_wr_data(bg_clr);
			                 }
			           }
		           tx <<= 1                ; // Shift Right data 1 bit
		           ay = ay-1               ; // -decrement Y-address
		         }
		    }
        // **** Sent data byte2,3,4=3x8bit (nur Font3) ****
        if  (font == 3)
		    {
			  for (uint8_t j=0; j<3; j++) // 3 Bytes anhngen fr diese spalte bei font3
			      {
			        m++                                      ; // Increment Next pointter byte Data
  		            tx = pgm_read_byte(&(ascii_22x32[row][m])); // Read data byte2
			        for(n=0;n<8;n++)		      // Loop Sent data  1 byte(8bit)
		               {
	                     if  (tx & 0x80)			  // if data bit7 = 1 ,Plot Color area Charecter
	 		                 {
	 		                   gp_wr_cmm(0x20)      ; // Command Set Adddress Hor(X)
                               gp_wr_data(ax)       ; // Sent X_Address CGRAM
                               gp_wr_cmm(0x21)      ; // Command Set Address Ver(Y)
	                           gp_wr_data(ay)       ; // Sent Y_Address CGRAM
				               gp_wr_cmm(0x22)      ; // Command Write data
                               gp_wr_data(fg_clr)   ;
 			                 }
		                 else{					        // if data bit7 = 0 ,Plot Color area back ground Charecter
			                   if  (bg_clr != 1)
                                   {
			                         gp_wr_cmm(0x20)   ; // Command Set Adddress Hor(X)
                                     gp_wr_data(ax)    ; // Sent X_Address CGRAM
                                     gp_wr_cmm(0x21)   ; // Command Set Adddress Ver(Y)
                                     gp_wr_data(ay)    ; // Sent Y_Address CGRAM
			                         gp_wr_cmm(0x22)   ; // Command Write data
                                     gp_wr_data(bg_clr); // Sent Data
			                       }
			                 }
		                 tx <<= 1                 ; // Shift Right data 1 bit
		                 ay   = ay-1              ; // Decrement Y-address
		               }
                  }
			}		
		// **** Sent data byte2,3,4,5,6,7=6x8bit (nur Font4) ****
        if  (font == 4)
		    {
			  for(uint8_t j =0; j<6; j++)
			     {
				   m++                                      ; // Increment Next pointter byte Data
  		           tx = pgm_read_byte(&(ascii_24x56[row][m])); // Read data byte2
			       for(n=0;n<8;n++)		      // Loop Sent data  1 byte(8bit)
		              {
	                    if  (tx & 0x80)			  // if data bit7 = 1 ,Plot Color area Charecter
	 		                {
	 		                  gp_wr_cmm(0x20)      ; // Command Set Adddress Hor(X)
                              gp_wr_data(ax)       ; // Sent X_Address CGRAM
                              gp_wr_cmm(0x21)      ; // Command Set Address Ver(Y)
	                          gp_wr_data(ay)       ; // Sent Y_Address CGRAM
				              gp_wr_cmm(0x22)      ; // Command Write data
                              gp_wr_data(fg_clr)   ;
 			                }
		                else{					        // if data bit7 = 0 ,Plot Color area back ground Charecter
			                  if  (bg_clr != 1)
                                  {
			                        gp_wr_cmm(0x20)   ; // Command Set Adddress Hor(X)
                                    gp_wr_data(ax)    ; // Sent X_Address CGRAM
                                    gp_wr_cmm(0x21)   ; // Command Set Adddress Ver(Y)
                                    gp_wr_data(ay)    ; // Sent Y_Address CGRAM
			                        gp_wr_cmm(0x22)   ; // Command Write data
                                    gp_wr_data(bg_clr); // Sent Data
			                      }
			                }
		                tx <<= 1                 ; // Shift Right data 1 bit
		                ay   = ay-1              ; // Decrement Y-address
		              }
                 }
			}		

		ax = ax+1; // Complet sent data 1,2 or 3 Bytes (8bit fr Font1, 11bit fr Font2, 24bit fr Font3. Eine Spalte fertig) Increment X-Address
		ay = ady ; // Set Position Y-address old
 	  };

   // **** Fill Back ground Color Position space between Charecter 1 Colum ****
   if  (font ==2)
	   {
	     if  (bg_clr != 1)
             {
	           for(n=0;n<11;n++)
	              {
	           		gp_wr_cmm(0x20)   ; // Command Set Adddress Hor(X)
               		gp_wr_data(ax)    ; // Sent X_Address CGRAM
               		gp_wr_cmm(0x21)   ; // Command Set Adddress Ver(Y)
               		gp_wr_data(ay)    ; // Sent Y_Address CGRAM
	           		gp_wr_cmm(0x22)   ; // Command Write data
               		gp_wr_data(bg_clr);
		       		ay = ay-1         ; // Incre ment Y-Address
	          	  }
             }
       }
  }
/************************************************
 Function Print String
 ************************************************
 Parameter :
 *str = Charecter ASCII (String)
 sx,sy = Position X,Y for begin plot text by will
 begin plot from bottom left to top left
 fg_color = color of Text
 bg_color = color Background of text
 ************************************************/
void print_Str(char *str,uint8_t font,uint8_t sx,uint16_t sy,uint16_t fg_color,uint16_t bg_color)
  {
   uint8_t i;
	for (i=0; str[i] != '\0'; i++)
	{
	 lcd_Ascii(str[i],font,sx,sy,fg_color,bg_color);
	 switch(font)
	       {
		     case 1: sx +=  6; break; // Abstand zwischen den Zeichen, bei Font1 =  6 pixel
             case 2: sx +=  8; break; // Abstand zwischen den Zeichen, bei Font2 =  8 pixel
			 case 3: sx += 22; break; // Abstand zwischen den Zeichen, bei Font3 = 22 pixel
			 case 4: sx += 25; break; // Abstand zwischen den Zeichen, bei Font4 = 24 pixel
           }
	}
  }

/*****************************************************
 Funktion: LCD Print Zahl
 *****************************************************
 Formatiert die bergebene Variable
 Sendet ber die LCD_Ascii Funktion eine maximal fnf-
 stellige Zahlenvariable an das Display.
 Parameter :
 - zahl    :die Zahlenvar. die ausgegeben werden soll
 - ziffern :Wieviele Ziffern? 1-5 Zifferstellen.
 - font    :Font fr die Ziffern.
 - x  	   :X-Position
 - y	   :Y-Position
 - farbe   :Ziffernfarbe
 - bgfarbe :Hintergrundfarbe
 *****************************************************/
void print_Zahl(uint16_t zahl,uint8_t ziffern,uint8_t font,uint8_t x,uint16_t y,uint16_t farbe, uint16_t bgfarbe)

  {
   uint8_t font_x   =0;			   // Variable fr die Fontbreite
   char	   ziffer   =0;			   // Variable fr die separierte Ziffer

   switch(font)
         {
		   case 1: font_x =  6; break; // X-Pos zur nchsten Ziffer an den Font anpassen
		   case 2: font_x =  8; break;
		   case 3: font_x = 22; break;
		   case 4: font_x = 25; break;
		 }

   for(char i=ziffern; i>0; i--)   // alle Ziffern der Zahl separieren
      {
	   if(zahl >9)				   // mehr als 1 Ziffer in Zahl?
	 	 {
	  	  ziffer =zahl %10;		   // Ziffer separieren und ...
	  	  zahl   =zahl /10;		   // von Zahl entfernen
	 	 }
   	   else						   // nur 1 Ziffer in Zahl?
     	 {
	  	  ziffer = zahl; zahl = 0; // Ziffer direkt aus Zahl entnehmen, nchste Ziffer =0
	 	 }
	   lcd_Ascii(ziffer +0x30,font,x+((i *font_x)-font_x),y,farbe,bgfarbe); // Ziffer am Display ausgeben
      }
  }

/*****************************
 Function Plot Dot 1 Pixel
 *****************************
 Parameter:
 dx,dy = Pos. X,Y for Plot Dot
 color = Color of dot
 *****************************/
void plot_dot(uint8_t dx,uint16_t dy,uint16_t color)
  {
   gp_wr_cmm(0x20)  ; // Command Set Adddress Hor(X)
   gp_wr_data(dx)   ; // Sent X_address for CGRAM
   gp_wr_cmm(0x21)  ; // Command Set Address Ver(Y)
   gp_wr_data(dy)   ; // Sent Y_address for CGRAM
   gp_wr_cmm(0x22)  ; // Command Write Data RGB
   gp_wr_data(color); // write Data
  }

/*********************************************
 Function Draw Line
 *********************************************
 Function: Line(lx1,ly1,lx2,ly2,line_type);
 Input: lx1,ly1   = Start Coordinate Line
        lx2,ly2   = End Coordinate Line
        line_type = Value Thick of Line(1-100)
	      color     = Color of line
 *********************************************/
  void Line(uint8_t lx1,uint16_t ly1,uint8_t lx2,uint16_t ly2,uint8_t line_type,uint16_t color)
   {
    uint8_t  tick      	   ;
    uint8_t  x1=lx1,xx1=lx1;
    uint16_t y1=ly1        ;
    uint8_t  x2=lx2,xx2=lx2;
    uint16_t y2=ly2        ;
	int16_t  dx,dy,inc_dec,two_dx,two_dy,two_dy_dx,two_dx_dy,x,y,p;
    if(lx1>lx2) {x1=lx2; y1=ly2; x2=lx1; y2=ly1;}
    dx  =abs(x2-x1); dy  =abs(y2-y1);
    if(y2>=y1) {inc_dec =1  ;}
	else       {inc_dec = -1;}
    if(dx>dy)
	  {
	   two_dy=(2*dy)         ;
	   two_dy_dx=(2*(dy-dx)) ;
	   p=((2*dy)-dx)         ;
       x=x1; y=y1;
       while(x<=x2)
		 {
	      for(tick=0;tick<line_type;tick++)
			 {
			  if(xx1==xx2){plot_dot(x+tick,y,color);}
			  else	      {plot_dot(x,y+tick,color);}
			  }
		  x++;
		    if(p<0) {p += two_dy;}
		    else    {y += inc_dec; p += two_dy_dx;}
		  } // while
	    } // if
       else
	    {
	     two_dx=(2*dx)        ;
	     two_dx_dy=(2*(dx-dy));
	     p=((2*dx)-dy)        ;
	      x=x1                ;
	      y=y1                ;
	     while(y!=y2)
		  {
		   for(tick=0;tick<line_type;tick++)
		    {
			 if(xx1==xx2) {plot_dot(x+tick,y,color);}
			 else		  {plot_dot(x,y+tick,color);}
			    }
			   y+=inc_dec;
		   if(p<0) {p+=two_dx;}
		   else    {x++; p += two_dx_dy;}
		 } // while
	   } // else
    }

/**************************************************************************
 Function Draw Circle
 **************************************************************************
 Function: Circle(cx,cy,Radius,tick,color,fill)
 Input   : CX,CY      = Center Point of Circle
           Radius     = radius of Circle
           tick(1-100)= bold of line
            - (have effect when fill=0)
           color      = color of Circle
           fill       = 0: No fill color in Circle, 1: Fill color in Circle
 **************************************************************************/
void Circle(int CX,int CY,unsigned char Radius,unsigned tick,unsigned int color,unsigned char fill)
  {
    int tswitch=0   ;
    unsigned char d,x=0,y=0,bd   ;


    d = CY-CX ;
	y = Radius ;
	tswitch = 3-(2*Radius);

	while(x<=y)	
	 {
	   if(fill==1)
	    {
		 Line(CX-x,CY+y,CX+x,CY+y,1,color) ;
		 Line(CX-x,CY-y,CX+x,CY-y,1,color) ;
		 Line(CX-y,CY+x,CX+y,CY+x,1,color) ;
		 Line(CX-y,CY-x,CX+y,CY-x,1,color) ;

		}
	   else
		{
	
		 for(bd=0;bd<tick;bd++)
		  {
		   plot_dot((CX+x)+bd,(CY+y)+bd,color)     ;
		   plot_dot((CX+x)+bd,(CY-y)+bd,color)     ;

		   plot_dot((CX-x)+bd,(CY+y)+bd,color)     ;
		   plot_dot((CX-x)+bd,(CY-y)+bd,color)     ;

		   plot_dot((CY+y-d)+bd,(CY+x)+bd,color)   ;
		   plot_dot((CY+y-d)+bd,(CY-x)+bd,color)   ;

		   plot_dot((CY-y-d)+bd,(CY+x)+bd,color)   ;
		   plot_dot((CY-y-d)+bd,(CY-x)+bd,color)   ;
		  }
	   	}

	   if(tswitch<0)
	     tswitch += (4*x)+6     ;
	   else
	    {
         tswitch += 4*(x-y)+10  ;
		 y--;
	    }
       x++;
     }              //while
	
  }

/************************************************
 Function Draw Ellipse
 ************************************************
 Function: Ellipse(h,k,rx,ry,tick,color,fill)
 Input: h,k        =Center Point of Ellipes
        rx,ry      =radius of Ellipse in X,Y axis
        tick(1-100)=bold of line
                   -(have effect when fill=0)
        color      =color of Ellipse
        fill       =0: No fill color in Ellipes,
                    1: Fill color in Ellipes
 ************************************************/
void Ellipse(uint16_t h,uint16_t k,long rx,long ry,uint8_t tick,uint16_t color,uint8_t fill)
    {
	  unsigned char bd 	   ;
      long  ax  =(rx*rx)   ; // ax  = halbehhe
      long   by  =(ry*ry)  ; // by  = halbebreite
      float  ax2 =(ax*2)   ; // ax2 = 2* halbehhe
      float  by2 =(by*2)   ; // by2 = 2* halbebreite
      long   x   = 0       ; // x = 0
      long   y   = ry      ; // y = halbebreite
      float  fx  = 0       ; // 
      float  fy  = (ax2*ry); // 2* halbehhe * halbebreite
      float     p=(int)(by-(ax*ry)+(0.25*ax)+0.5); // p = halbebreite-(halbehhe * halbebreite)+(0.25 * halbehhe)+0.5)
	   if(fill==1)
	    {
		 Line(h-x,k+y,h+x,k+y,1,color);
		 Line(h-x,k-y,h+x,k-y,1,color);
	    }
	   else
	    {
		 for(bd=0;bd<tick;bd++) // fr die liniendicke
		  {
           plot_dot((h+x)+bd,(k+y)+bd,color);
		   plot_dot((h+x)+bd,(k-y)+bd,color);
		   plot_dot((h-x)+bd,(k-y)+bd,color);
		   plot_dot((h-x)+bd,(k+y)+bd,color);
		  }
		}

       while(fx<fy)
	   {
	     x++;
	     fx+=by2;

	     if(p<0)
		p+=(fx+by);

	     else
		{
		   y--;
		   fy-=ax2;
		   p+=(fx+by-fy);
		}

		if(fill==1)
	    {
		 Line(h-x,k+y,h+x,k+y,1,color);
		 Line(h-x,k-y,h+x,k-y,1,color);
	    }
	   else
	    {
		  for(bd=0;bd<tick;bd++)
		   {
		    plot_dot((h+x)+bd,(k+y)+bd,color);
		    plot_dot((h+x)+bd,(k-y)+bd,color);
		    plot_dot((h-x)+bd,(k-y)+bd,color);
		    plot_dot((h-x)+bd,(k+y)+bd,color);
		   }
		}
	  }
       p=(int)((by*(x+0.5)*(x+0.5))+(ax*(y-1)*(y-1))-(ax*by)+0.5);
       while(y>0)
	  {
	     y--;
	     fy-=ax2;

	     if(p>=0)
		p+=(ax-fy);
	     else
		{
		   x++;
		   fx+=by2;
		   p+=(fx+ax-fy);
		}
		if(fill==1)
	    {
		 Line(h-x,k+y,h+x,k+y,1,color);
		 Line(h-x,k-y,h+x,k-y,1,color);
	    }
	   else
	    {
		  for(bd=0;bd<tick;bd++)
		   {
		    plot_dot((h+x)+bd,(k+y)+bd,color);
		    plot_dot((h+x)+bd,(k-y)+bd,color);
		    plot_dot((h-x)+bd,(k-y)+bd,color);
		    plot_dot((h-x)+bd,(k+y)+bd,color);
		   }
		}
	  }
    }

/************************************************
 Funktion Draw a rectangle
 ************************************************
 Function: Rectan(x1,y1,x2,y2,tick,color,fill)
 Inputs  :(x1, y1) =Start Point coordinate Rectan
          (x2, y2) =End Point coordinate  Rectan
       tick(1-100) =bold of line(fx when fill=0)
       color       =color of rectangle
       fill        =0:No fill color in rectangle
                    1:Fill color in rectangle

       (x1,y1)+-------------+
              | -----> x    |
              ||            |
              ||            |
              |y            |
              +-------------+	(x2,y2)
 ************************************************/
void Rectan(uint8_t x1,uint16_t y1,uint8_t x2,uint16_t y2,uint8_t tick,uint16_t color,uint8_t fill)
  {
   uint16_t xmin,xmax,ymin,ymax,i;
   uint8_t tk = tick-1           ;
     if(fill==1)
	  {
	       if(x1 < x2)        // Find x min and max
              {
               xmin = x1;
               xmax = x2;
			  }
			 else
			  {
			    xmin = x2;
			    xmax = x1;
			   }
			 if(y1 < y2)      // Find the y min and max
			   {
			     ymin = y1;
			     ymax = y2;
			   }
			 else
			   {
			    ymin = y2;
			    ymax = y1;
			   }
			 for(; xmin<=xmax;xmin++)
			   {
			     for(i=ymin; i<=ymax; i++)
				   plot_dot(xmin,i,color);
			   }
	  }
	 else	// No-Fill
	  {
	    Line(x1,y1,x2,y1,tick,color)      ;
	    Line(x1,y2-tk,x2,y2-tk,tick,color);
	    Line(x1,y1,x1,y2,tick,color)      ;
	    Line(x2-tk,y1,x2-tk,y2,tick,color);
	  }
  }

/****************************************
 Function Plot Mark(+) fr Kalibrierung
 ****************************************
 Parameter:
     x1,y1 = Position start Line '-'(Hor)
             plot from Left to Right
     x2,y2 = Position start Line '|'(Ver)
             plot from botton to top
 ****************************************/
void plot_mark(uint8_t x1,uint16_t y1,uint8_t x2,uint16_t y2,uint16_t color)
  {
   uint8_t i		       ;
  // **** Line- ****
   gp_wr_cmm(0x20)         ; // Command Set Adddress Hor(X)
   gp_wr_data(x1)          ; // Sent X_address  CGRAM
   gp_wr_cmm(0x21)         ; // Command Set Address Ver(Y)
   gp_wr_data(y1)          ; // Sent Y_address  CGRAM
   gp_wr_cmm(0x22)         ; // Command Write Data RGB
   for(i=0;i<15;i++)
       gp_wr_data(color)   ;
  // ****** Line | *******
   gp_wr_cmm(0x03)         ; // Command Configure Entry Mode
   gp_wr_data(0x1238)      ; // Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.
   gp_wr_cmm(0x20)         ; // Command Set Adddress Hor(X)
   gp_wr_data(x2)          ; // Sent X_Address CGRAM
   gp_wr_cmm(0x21)         ; // Command Set Address Ver(Y)
   gp_wr_data(y2)          ; // Sent Y_Address CGRAM
   gp_wr_cmm(0x22)         ; // Command Write Data RGB
   for(i=0;i<15;i++)
       gp_wr_data(color)   ;
   gp_wr_cmm(0x03)         ; // Command Configure Entry Mode
   gp_wr_data(0x1230)      ; // Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.
  }



