/********************************************************************
 UART Ini aus der PDF zum ETT Atmega2560 Modul
 ********************************************************************/
#include <avr/io.h>
#include <stdio.h>
#include <avr/pgmspace.h>

#define extern
#include "uart_routines.h"
#undef extern

//#define F_CPU 16000000UL //16.0 MHz im Configurationsfenster eingestellt 16000000
#define BAUD 9600 //9600 BPS
#define MYUBRR F_CPU/16/BAUD-1


void init_serial(void); //Initil UART

unsigned char receiveByte(void);
void transmitByte(unsigned char);
void transmitString_F(char*);
void transmitString(unsigned char*);
void transmitHex( unsigned char dataType, unsigned long data );




/****************/
/* Initial UART */
/****************/
void init_serial(void) //Initilial UART0 = 9600,N,8,1
 {
   unsigned int ubrr = MYUBRR;
   UBRR0H = (unsigned char)(ubrr>>8); //Set baud rate
   UBRR0L = (unsigned char)ubrr;
   UCSR0B = (1<<RXEN0)|(1<<TXEN0); //Enable RX and TX
   UCSR0C = (1<<USBS0)|(3<<UCSZ00); //8data, 2stop bit
 }

/*****************************/
/* Get 1 Character From UART */
/*****************************/
unsigned char receiveByte(void)
 {
   while(!(UCSR0A & (1<<RXC0))); // Wait RXD Receive Data Ready
   return (UDR0); // Get Receice Data & Return
 }

/*****************************/
/* Write 1 Character To UART */
/*****************************/
void transmitByte( unsigned char data)
 {
   while (!(UCSR0A & (1<<UDRE0))); // Wait TXD Buffer Empty
   UDR0 = data;
 }

//***************************************************
//Function to transmit hex format data
//first argument indicates type: CHAR, INT or LONG
//Second argument is the data to be displayed
//***************************************************
void transmitHex( unsigned char dataType, unsigned long data )
{
unsigned char count, i, temp;
unsigned char dataString[] = "0x        ";

if (dataType == CHAR) count = 2;
if (dataType == INT) count = 4;
if (dataType == LONG) count = 8;

for(i=count; i>0; i--)
{
  temp = data % 16;
  if((temp>=0) && (temp<10)) dataString [i+1] = temp + 0x30;
  else dataString [i+1] = (temp - 10) + 0x41;

  data = data/16;
}

transmitString (dataString);
}

//***************************************************
//Function to transmit a string in Flash
//***************************************************
void transmitString_F(char* string)
{
  while (pgm_read_byte(&(*string)))
   transmitByte(pgm_read_byte(&(*string++)));
}

//***************************************************
//Function to transmit a string in RAM
//***************************************************
void transmitString(unsigned char* string)
{
  while (*string)
   transmitByte(*string++);
}






