
/*********************************************
 *            AVR Sound Treiber              *
 *                                           *
 *   fr einfachen Buzzer direkt am Port     *
 *   PE3 wird getoggelt um einen Ton am      *
 *   angeschlossenen Speaker zu erzeugen.    *
 *                                           *
 *********************************************
 *                                           *
 *  PE3 (OC3A) - Buzzer gegen GND            *
 *  Timer3 wird fr die Tonerzeugung genutzt *
 *                                           *
 *********************************************/
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>	// Interrupts


#define extern
#include "sound.h"
#undef extern




void init_sound(void)
  {
   // Setup PE3 as an output
   DDRE = _BV(PE3);
   // Set the Timer/Counter Control Register
   TCCR3A = (1<<COM3A1)|(1<<COM3A0); // Setze nach dem hochzhlen das Bit OC3A(PE3) auf 1, clear when down counting
   TCCR3B = (1<<WGM33);              // Phase/Freq-correct PWM, top value = ICR3
    // Lautstrke stellen
    OCR3AH = 0;
    OCR3AL = 255;
  };

void play_note(unsigned int note,unsigned int duration)
  { // Reset the 16 bit Counter
    TCNT3H = 0;
    TCNT3L = 0;

    // Set the Counter TOP
    ICR3H = ( note >> 8 ) & 0x00FF;
    ICR3L = note;

    // Turn on the Prescaler
    TCCR3B |= (1<<CS31);
    _delay_ms(duration);

    // Turn off the Prescaler
    TCCR3B &= ~(1<<CS31);
    _delay_ms(PAUSE);
  };

void play_melody(unsigned int typ)
  { 
    switch(typ)
	   {        // Beep1
	     case  1: play_note(C7, 50);
		 break; // Beep2
		 case  2: play_note(A6, 50);
		 break; // Beep3
		 case  3: play_note(F6, 50);
		 break; // Beep4
		 case  4: play_note(D6, 50);
		 break; // doppelBeep1
		 case  5: play_note(C7, 100);
		          play_note(C7, 100);
         break; // doppelBeep2
		 case  6: play_note(C6, 50);
		          _delay_ms(25);
		          play_note(C6, 50);
	     break; // Twinkle Star
		 case 10: play_note(C6,200); // print_Str("Twin",2,30 ,50 ,0xFFF3,no_bg);
	              play_note(C6,200); // print_Str("kle" ,2,58 ,50 ,0xFFF3,no_bg);
	              play_note(G6,200); // print_Str("Twin",2,86 ,50 ,0xFFF3,no_bg); 
	              play_note(G6,200); // print_Str("kle" ,2,114,50 ,0xFFF3,no_bg);  

	              play_note(A6,200); // print_Str("Lit" ,2,30 ,75 ,0xFFF3,no_bg);
                  play_note(A6,200); // print_Str("tle" ,2,51 ,75 ,0xFFF3,no_bg);
	              play_note(G6,500); // print_Str("Star",2,79 ,75 ,0xFFF3,no_bg);

	              play_note(F6,200); // print_Str("How" ,2,30 ,100,0xFFF3,no_bg);
	              play_note(F6,200); // print_Str("I"   ,2,58 ,100,0xFFF3,no_bg);
	              play_note(E6,200); // print_Str("Won" ,2,72 ,100,0xFFF3,no_bg);
                  play_note(E6,200); // print_Str("der" ,2,93 ,100,0xFFF3,no_bg);

                  play_note(D6,200); // print_Str("What",2,30 ,125,0xFFF3,no_bg);
                  play_note(D6,200); // print_Str("You" ,2,65 ,125,0xFFF3,no_bg);
	              play_note(C6,800); // print_Str("Are!",2,93 ,125,0xFFF3,no_bg);
         break;
       };
  };

