
 /************************************************************************
 *      Taster Entprellung fr bis zu 8 Tasten an gleichen Port          *
 *                                                                       *
 *                               *Up                                     *
 *         *Escape       *Left         *Right        *Enter              *
 *                               *Down                                   *
 *                                                                       *
 *************************************************************************
 * PJ0: Escape Taste gegen GND                                           *
 * PJ1: Links Taste  gegen GND                                           *
 * PJ2: Hoch Taste   gegen GND                                           *
 * PJ3: Runter Taste gegen GND                                           *
 * PJ4: Rechts Taste gegen GND                                           *
 * PJ5: Enter Taste  gegen GND                                           *
 *************************************************************************
 *                                                                       *
 *                      Debouncing 8 Keys                                *
 *                      Sampling 4 Times                                 *
 *                      With Repeat Function                             *
 *                                                                       *
 *              Author: Peter Dannegger                                  *
 *                      danni@specs.de                                   *
 *                                                                       *
 ************************************************************************/
 
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
 
#define extern
#include "../update.h"
#include "taster.h"
#include "../Kontakte/kontakte.h"     // Reedkontakte, Rauch,-Bewegungsmelder usw.
#undef extern



void init_taster( void )
{
  // Configure debouncing routines
  KEY_DDR &= ~ALLE_TASTER;                // configure key port for input
  KEY_PORT |= ALLE_TASTER;                // and turn on pull up resistors
 
  // Timer0 Overflow fr die Tasten Entprellung einstellen
  TCCR2B = (1<<CS22)|(1<<CS20);         // divide by 1024
  TCNT2 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);  // preload for 10ms
  TIMSK2 |= 1<<TOIE2;                   // enable timer interrupt

  sei();                // Globale Interrupts einschalten

};



ISR( TIMER2_OVF_vect )                            // every 10ms
{
  static uint8_t ct0, ct1, rpt;
  uint8_t i;
 
  TCNT2 = (uint8_t)(int16_t)-(F_CPU / 1024 * 10e-3 + 0.5);  // preload for 10ms
 
  // key changed ?
  //i = key_state ^ KEY_PIN; // fr activehighTaster
  i = key_state ^ ~KEY_PIN; // fr activeLowTaster 

  ct0 = ~( ct0 & i );                             // reset or count ct0
  ct1 = ct0 ^ (ct1 & i);                          // reset or count ct1
  i &= ct0 & ct1;                                 // count until roll over ?
  key_state ^= i;                                 // then toggle debounced state
  key_press |= key_state & i;                     // 0->1: key press detect
 
  if( (key_state & REPEAT_MASK) == 0 )            // check repeat function
     rpt = REPEAT_START;                          // start delay
  if( --rpt == 0 ){
    rpt = REPEAT_NEXT;                            // repeat delay

  // key_rpt = ~(key_rpt | (key_state & REPEAT_MASK)); // fr activeHighTaster
  key_rpt |= key_state & REPEAT_MASK; // bei active_low Tastern

  // Kontakt Port Abfrage alle 10ms
  if  (Kontakte_Port != PINK) {kontakt_update();}
  // Bedingung fr eine Aktor Zustands-nderung eingetreten?
  //***//hier code hin
  }
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
uint8_t get_key_press( uint8_t key_mask )
{
  cli();                                          // read and clear atomic !
  key_mask &= key_press;                          // read key(s)
  key_press ^= key_mask;                          // clear key(s)
  sei();
  return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported being pressed in subsequent calls
// to this function. This simulates the user repeatedly
// pressing and releasing the key.
//
uint8_t get_key_rpt( uint8_t key_mask )
{
  cli();                                          // read and clear atomic !
  key_mask &= key_rpt;                            // read key(s)
  key_rpt ^= key_mask;                            // clear key(s)
  sei();
  return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
uint8_t get_key_short( uint8_t key_mask )
{
  cli();                                          // read key state and key press atomic !
  return get_key_press( ~key_state & key_mask );
}

///////////////////////////////////////////////////////////////////
//
uint8_t get_key_long( uint8_t key_mask )
{
  return get_key_press( get_key_rpt( key_mask ));
}
 



