/* Menu Buttons.C */

// AVR interne Module;
//#include <avr/pgmspace.h> // strcat... 
#include <avr/interrupt.h>
#include <stdint.h>
#include <avr/eeprom.h>
#include <string.h> 
#include <stdlib.h> // itoa ...

// Projekt-Ordner Module;
#define extern
#include "msr_unit01.h"      // 
#include "update.h"          // update Header
#include "debug.h"           // abstellbarer Debugmodus fr die Fehlersuche
#include "buttons.h"         // Header
#include "funktionen.h"      // Menu Funktionen
#include "mmc_funktionen.h"  // MMC Funktionen
#include "ETT-Display/ett240320tp32revb.h" // ET-TFT Display Funktionen
#include "Clock/clock.h"      // DCF - Uhr Header
#include "Clock/clock_draw.h" // Analog Uhr Koordinaten
#include "SD_Card/fat32.h"    // FAT32 Funktionen
#include "SD_Card/uart_routines.h" // UART Terminal ausgabe Funktionen


#undef extern


/**************************************
 Funktion build Button
 **************************************
 Parameter:

   xleft,ytop ->+----------------+---+
                |                |   A
                |                |	 |
                |     (farbe)    | hhe
                |                |	 |
                |                |	 V
                +----------------+---+
				|<--- breite --->|

 **************************************/
void button_build(uint8_t xleft,uint16_t ytop,uint8_t width,uint16_t height,uint8_t color)
 {
   uint16_t xright,ydown,ymiddle;
   uint8_t range;

   color_set(color); // Farbe 1 bis 31
   uint16_t clrsub = Menu.Farbe.Sub;
   uint16_t dark   = Menu.Farbe.Dunkel;
   uint16_t bright = Menu.Farbe.Hell;
   uint16_t clrfoo = bright;

   xright  =xleft + width -1;
   ydown   =ytop + height -1;
   ymiddle =ytop +(height/2);

// --- Start Plot halft button from botton to top ---
   for (range=1+(height/2); range>0; range--)
       {
         Line(xleft,ydown,xright,ydown,1,bright);
         ydown --;
         bright =bright -clrsub;
	     if(bright<dark) {bright =dark;}
       }

//  --- Plot halft button from top to botton ---
   bright = clrfoo;
   for (range=1+(height/2); range>0; range--)
       {
         Line(xleft,ytop,xright,ytop,1,bright);
         ytop ++;
         bright = bright -clrsub;
	     if(bright < dark) {bright =dark;}
       }
  }



/**********************************************************
 Funktion Menleiste Button; 239breit
 **********************************************************/
 void leiste_btn(void)
  {
    color_set(RAM_Data.Farbe.Leiste);
    button_build(  2, 2, 77, 21, RAM_Data.Farbe.Leiste);
    button_build(161, 2, 77, 21, RAM_Data.Farbe.Leiste);

    switch(Menu.Window)
          { 
	        case Kal_Menu:   // --- Touch Kalibrierung Men ---
	             print_Str("neu starten", 1,   7, 16, Menu.Farbe.Text, no_bg);
		         print_Str(" Kali "     , 2,  96, 16, Menu.Farbe.Text, no_bg);
                 print_Str(" Hauptmen} ", 1, 166, 16, Menu.Farbe.Text, no_bg);
            break;
	        case Haupt_Menu: // --- Hauptmen ---
			        print_Str("Touch Kalib", 1,   7, 16, Menu.Farbe.Text, no_bg);
	  		        print_Str(" Men} ",      2,  96, 16, Menu.Farbe.Text, no_bg);
  		            print_Str("Pumpen Men}", 1, 166, 16, Menu.Farbe.Text, no_bg);
            break;
			case Pumpen_Menu: // --- Pumpen Men ---
		            print_Str(" Hauptmen} ", 1,   7, 16, Menu.Farbe.Text, no_bg);
   	  		        print_Str("Pumpen",      2,  96, 16, Menu.Farbe.Text, no_bg);
				    print_Str("   frei    ", 1, 166, 16, Menu.Farbe.Text, no_bg);
		    break;
	        case Relais_Menu: // --- Relaismen ---
		            print_Str(" Hauptmen} ", 1,   7, 16, Menu.Farbe.Text, no_bg);
   	  	            print_Str("Aktion",      2,  96, 16, Menu.Farbe.Text, no_bg);
  		            print_Str("RelaisListe", 1, 166, 16, Menu.Farbe.Text, no_bg);
            break;
	        case Relais_Liste: // --- Relais Listenmen ---
		  	        print_Str("   Relais  ", 1,   7, 16, Menu.Farbe.Text, no_bg);
	  		        print_Str("Liste",       2, 100, 16, Menu.Farbe.Text, no_bg);
  		            print_Str(" Hauptmen} ", 1, 166, 16, Menu.Farbe.Text, no_bg);
            break;
	        case Info_Menu: // --- Info & Meldungen Men ---
			        print_Str(" Hauptmen} ", 1,   7, 16, Menu.Farbe.Text, no_bg);
	  		        print_Str(" Info ",      2,  96, 16, Menu.Farbe.Text, no_bg);
  		            print_Str("   frei    ", 1, 166, 16, Menu.Farbe.Text, no_bg);
            break;
	        case Kalender_Menu: // --- Kalender Men ---
			        print_Str(" Hauptmen} ", 1,   7, 16, Menu.Farbe.Text, no_bg);
	  		        print_Str(" Zeit ",      2,  96, 16, Menu.Farbe.Text, no_bg);
  		            print_Str("   frei    ", 1, 166, 16, Menu.Farbe.Text, no_bg);
            break;

          }
  }



/***************************************************************************
 Funktion Anzeige Feld im Hauptmen;

 Parameter[in]:
 - press: 0 Anzeige erstellen
          1 Anzeige Feld gedrckt (nur fr typ 0,1,2)
		  2 Anzeige aktualisieren (typ 0,1,2 nur aktive Felder)

 Abmessung: Digitalfelder
  y=25 ->+--------+--------+--------+---------   y=25 ->+--------+--------+---------  y=25 ->+---------+----------
         | 1.Feld | 2.Feld | 3.Feld |height=40          |        |        |             	 |         |
  y=68 ->+--------+--------+--------+---------          | 1.Feld | 2.Feld |height=61         |         |
         | 4.Feld | 5.Feld | 6.Feld |height=40   y=89 ->+--------+--------+---------         |  1 Feld |height=127
 y=111 ->+--------+--------+--------+---------          |        |        |                  |         |
         | 7.Feld | 8.Feld | 9.Feld |height=40          | 3.Feld | 4.Feld |height=61         |         |
         +--------+--------+--------+---------   	    +--------+--------+---------    	 +---------+---------- 
         ^<--77-->^<--78-->^<--77-->                    ^<--117->^<--117->		             ^<--236-->
         x=2      x=81     x=161                        x=2      x=121                       x=2
 
 
 Abmessung: Chart Grafik
 - Bechriftung(Wert)-------->y= 33 (X)Wert "Wert-Typ"               
 - Rand oben (Wert)Skala---->y= 36 +---+------------------------+  +--+<-Y=34
 - Max.punkt (Wert)Skala---->y= 37 |  -|                        |  |->| next chart Btn (height 49)
						          (X) -|                        |  +--+<-Y=83
 - Nullpunkt (Wert)Skala---->y=137 |  -|     <- (Y)Zeit ->      |  +--+<-Y=85
 - Rand unten (Wert)Skala--->y=138 +---+------------------------+  |<-| last chart Btn (height 49)
 - feine (Zeit)skalierung--->y=139     ||||||||||||||||||||||||||  +--+<-Y=134
 - grobe (Zeit)skalierung--->y=140     || | | | | | | | | | | | |  
 - Ziffern (Zeit)skalierung->y=141     ||   |   |   |   |   |   |  
 - Beschriftung(Zeit)------->y=151     |0   4   8   12  16  20  |  
                                       ^                        ^  
                          Tages Chart x=45 (191pixel 24h)     x=
                          WochenChart x=55 (177pixel 1Woche)  x=
                          MonatsChart x=45 (184pixel 1Monat)  x=
                          JahresChart x=45 (190pixel 1Jahr)   x=

 Abmessung: Liste
 - Feld Start Oben---------->y= 25 (font2)(offen/zu oder Ein/Aus)
 - 1.Textzeile-------------->y= 41 |24.12.2012 |20:12:55 |zu    |
 - ListenZeile-------------->y= 44 +-----------+---------+------+
 - 2.TextZeile-------------->y= 56 |24.12.2012 |20:14:56 |offen |
 - Listenzeile-------------->y= 59 +-----------+---------+------+
 - 3.TextZeile-------------->y= 71 |24.12.2012 |21:15:54 |zu    |
 - ListenZeile-------------->y= 74 +-----------+---------+------+
                                     ...
 - Listenzeile-------------->y=134 +-----------+---------+------+
 - 8.TextZeile-------------->y=146 |01.01.2013 |01:01:00 |offen |
 - Listenzeile-------------->y=149 +-----------+---------+------+
                                   ^           ^         ^      ^
                                  x=9       x=101     x=177    x=220
								    ^           ^         ^
						  textstartX=11        X=103      X=179
 ***************************************************************************/
void ausgabe_fld(uint8_t press)
  {
    color_set(Hintergrund);
	uint16_t gnd_color= Menu.Farbe.Grund;
	uint8_t xleft=2;

    // Dig.Felder A/D Clock
    color_set(RAM_Data.AnzOp1 +1);
	uint16_t fld_color = Menu.Farbe.Dunkel+(5*Menu.Farbe.Sub); // Feldfarbe
    uint16_t txt_color = txt_clr(RAM_Data.AnzOp2);             // Textfarbe
		          
    uint8_t width =77;
	uint8_t height =31;
    uint16_t ytop =25;

	uint8_t hh_foo;

	uint8_t start  =0; // fr 9 digit Felder
	uint8_t felder =9; // "
	uint8_t font   =1; // "
    uint8_t abstand =12; // "
    
	switch(press)
	      { case 1: // --- Press ---
		            switch(RAM_Data.AnzTyp)
		                  {
						    case 0: // --- 9 digit Felder gedrckt ---
				                    if  (Menu.Tab==4 || Menu.Tab==7 || Menu.Tab==10) {xleft = 81; width =78;}
					                if  (Menu.Tab==5 || Menu.Tab==8 || Menu.Tab==11) {xleft =161;}
			                        if  (Menu.Tab >5)
									    { if  (Menu.Tab <9) {ytop = 68;}
			                              else              {ytop =111;}
					                    }
						           	// Digital Feld Einstellung speichern
				                    RAM_Data.Feld[Menu.Tab -3].SenTyp = RAM_Data.SenTyp;
			                        RAM_Data.Feld[Menu.Tab -3].SenNum = RAM_Data.SenNum;
									RAM_Data.Feld[Menu.Tab -3].AnzOp1 = RAM_Data.AnzOp1;
			                        RAM_Data.Feld[Menu.Tab -3].AnzOp2 = RAM_Data.AnzOp2;
			                        cli();
				  		            eeprom_write_block(&RAM_Data.Feld[Menu.Tab-3],&EEP_Data.Feld[Menu.Tab-3],sizeof(RAM_Data.Feld[Menu.Tab-3]));
			      	                sei();
						   			// Digitial Feld erstellen
					       			Rectan(xleft, ytop, xleft+width-1, ytop+40, 0, fld_color, 1);
						   			if  (RAM_Data.SenTyp >0)
									    {label_value(0, RAM_Data.SenTyp, RAM_Data.SenNum, 1, xleft+11, ytop+12, txt_color, no_bg);}
			                break;
			                case 1: // --- 4 digit Felder gedrckt ---
									width =117;
					                if  (Menu.Tab==4 || Menu.Tab==6) {xleft =121;}
			                        if  (Menu.Tab >4) {ytop = 89;}

						         	// Digital Feld Einstellung speichern
				                    RAM_Data.Feld[Menu.Tab +6].SenTyp = RAM_Data.SenTyp;
			                        RAM_Data.Feld[Menu.Tab +6].SenNum = RAM_Data.SenNum;
									RAM_Data.Feld[Menu.Tab +6].AnzOp1 = RAM_Data.AnzOp1;
			                        RAM_Data.Feld[Menu.Tab +6].AnzOp2 = RAM_Data.AnzOp2;
			                        cli();
				  		            eeprom_write_block(&RAM_Data.Feld[Menu.Tab+6],&EEP_Data.Feld[Menu.Tab+6],sizeof(RAM_Data.Feld[Menu.Tab+6]));
			      	                sei();
						   			// Digital Feld neu erstellen
					       			Rectan(xleft, ytop, xleft+width-1, ytop+61, 0, fld_color, 1);
						   			if  (RAM_Data.SenTyp >0)
									    {label_value(0, RAM_Data.SenTyp, RAM_Data.SenNum, 2, xleft+32, ytop+20, txt_color, no_bg);}
			                break;
			                case 2: // --- 1 digit Feld gedrckt ---
									width =236;

								    // Digital Feld Einstellung speichern
				                    RAM_Data.Feld[13].SenTyp = RAM_Data.SenTyp;
			                        RAM_Data.Feld[13].SenNum = RAM_Data.SenNum;
								    RAM_Data.Feld[13].AnzOp1 = RAM_Data.AnzOp1;
			                        RAM_Data.Feld[13].AnzOp2 = RAM_Data.AnzOp2;
			                        cli();
				  		            eeprom_write_block(&RAM_Data.Feld[13],&EEP_Data.Feld[13],sizeof (RAM_Data.Feld[13]));
			      	                sei();
						   			// Digital Feld neu erstellen
					       			Rectan(xleft, ytop, xleft+width-1, ytop+126, 0, fld_color,1);
						   			if  (RAM_Data.SenTyp >0)
									    {label_value(0, RAM_Data.SenTyp, RAM_Data.SenNum, 3, 28, 65, txt_color, no_bg);}
			                break;
							default:// --- Chart Skala Up/Down gedrckt ---
			                        if  (RAM_Data.SenTyp >2 && RAM_Data.SenTyp <10 && RAM_Data.AnzTyp >2 && RAM_Data.AnzTyp <7)
									    { if  (Menu.Tab==3 || Menu.Tab==4)
							                  { // --- Chart Skala hoch/runter ---
									       	    if  (Menu.Tab==3)
											        { if  (RAM_Data.SenTyp==5)
														  { // C Chart mit Minuswerten bekommt ne Sonderbehandlung ...
													        if  (RAM_Data.AnzOp3 <11)
														        {RAM_Data.AnzOp3 +=1;}
														    else{ if  (RAM_Data.AnzOp3 ==12) {RAM_Data.AnzOp3  =0;}
														          else                       {RAM_Data.AnzOp3 -=1;}
			                                                    }
			                                              }
			                                          else{ if  (RAM_Data.AnzOp3 <15) {RAM_Data.AnzOp3 +=1;}
			                                              }
			                                        }
			                                    else{ if  (RAM_Data.SenTyp==5)
													      { if  (RAM_Data.AnzOp3 > 0)
														        { if  (RAM_Data.AnzOp3 <12) {RAM_Data.AnzOp3 -=1;}
			                                                      else{ if  (RAM_Data.AnzOp3 <15) {RAM_Data.AnzOp3 +=1;}
			                                                          }
			                                                    }
			                                                else{ RAM_Data.AnzOp3 =12;}
			                                              }
			                                          else{ if  (RAM_Data.AnzOp3 > 0) {RAM_Data.AnzOp3 -=1;}
													      }
			                                        }
												press=0;
			                                  }
			                            }
			                break;
			              }
            break;
			case 0: // --- Build ---
				    Rectan(0, 23, 239, 152, 0, gnd_color, 1); // Anzeige komplett lschen
	                if  (RAM_Data.AnzTyp <3)
		                { // [Digital Felder]
				          switch(RAM_Data.AnzTyp)
				                { case 1: start = 9; felder = 4; font = 2; abstand =23; break; // 4 digit Felder
				                  case 2: start =13; felder = 1; font = 3; abstand =32; break; // 1 digit Feld
                                }
         
		                  for (uint8_t i =start; i<start +felder; i++)
             	              { // X und breite einstellen
               	                if  (i==0 || i==3 || i==6)                   {xleft =  2; width = 77;} // 9 F
	           	                else{ if  (i==1 || i==4 || i==7)             {xleft = 81; width = 78;} // 9 F
					                  else{ if  (i==2 || i==5 || i==8)       {xleft =161; width = 77;} // 9 F
				      	                    else{ if  (i== 9 || i==11)       {xleft =  2; width =117;} // 4 F
			                  	                  else{ if  (i==10 || i==12) {xleft =121; width =117;} // 4 F   
                                                        else                 {xleft =  2; width =236;} // 1 F
                                      	              }
                               		            }
						 	              }
                   		            }
               		            // Y einstellen
 		       		            if  (i<3)                          {ytop =  25; height = 40;} // 9F 1.reihe
		       		            else{ if  (i<6)                    {ytop =  68; height = 40;} // 9F 2.reihe
			        	              else{ if  (i<9)              {ytop = 111; height = 40;} // 9F 3.reihe
						                    else{ if  (i<11)       {ytop =  25; height = 61;} // 4F 1.reihe
			              	                      else{ if  (i<13) {ytop =  89; height = 61;} // 4F 2.reihe
				               	                        else       {ytop =  25; height =126;} // 1F 
                                                      }
                              	                }
                         	              }
                   		            }
			   	                // Digital Feld erstellen
								color_set(RAM_Data.Feld[i].AnzOp1); // farbe einstellen
                                fld_color = Menu.Farbe.Dunkel+(5*Menu.Farbe.Sub);
					            Rectan(xleft, ytop, xleft+width-1, ytop+height, 0, fld_color, 1); // Feld erstellen
			   		            
								if  (RAM_Data.Feld[i].SenTyp)
					                { // Feld beschriften, Label oben
									  label_value(0, RAM_Data.Feld[i].SenTyp, RAM_Data.Feld[i].SenNum,
									  font, xleft+11,ytop+abstand, txt_clr(RAM_Data.Feld[i].AnzOp2), no_bg);
				   		            }
              	              }
                        }
                    else{ if  (RAM_Data.AnzTyp <7)
		                      { // [Chart/ Liste]
					            if  (RAM_Data.SenTyp <2)
					                {print_Str("<  Sensor Typ w{hlen!  >",2, 24, 93, txt_color, no_bg);}
                                else{ 
								      uint8_t load;
					                  mach_dateiname(1, RAM_Data.SenTyp, RAM_Data.SenNum, RAM_Data.AnzTyp); // Datei Name zum laden erzeugen

					   	              if  (RAM_Data.SenTyp <10)
							              { // --- Charts --- ... ist der akt. Sensorwert gltig?
                                            if  (Menu.Value[array_zu_sentyp(2, RAM_Data.SenTyp, RAM_Data.SenNum)] > -32000)
								                { 
												  if  (readFile (VERIFY, fileName)==1 || !RAM_Data.AnzOp1)
						                              { // existiert eine Datei oder ist der aktuelle Tag/Woche/Monat/Jahr ausgewhlt fr den evtl noch keine datei angelegt wurde
											            symbole(0, 6, 53, txt_color); // Skala Nullpunkt+ Btn erstellen
											            symbole(2, 6,112, txt_color); // Skala Nullpunkt- Btn erstellen
			                      			            Menu.bit16_1= -32000; // Min non
					                                    Menu.bit16_2= -32000; // Max non
											            print_Str("lade Chart...",1,72,33,txt_color, no_bg);
										                switch(RAM_Data.AnzTyp) // Skala (X)Zeitachse erstellen
											                  {
													            case 3: // Tageschart (24*8pix =192 breit)
				                                                        xleft =45; // X-Pos fr Y-Skala Werte
													                    for (uint8_t i=xleft;i<237;i+= 2) {plot_dot(i,139,txt_color);} // feine Skala  (1/4 h)
				                                                        for (uint8_t i=xleft;i<237;i+= 8) {plot_dot(i,140,txt_color);} // grobe Skala  (1 h)
                                                                        for (uint8_t i=xleft;i<236;i+=16) {plot_dot(i,141,txt_color);} // Ziffer Skala  (2 h)
				                                                        for (uint8_t i=0; i<12; i++)
				                                                            { if  (i<5) {print_Zahl( i*2, 1, 1, ((i*16)+42), 151, txt_color, no_bg);} // 0 Uhr bis
                                                                              else      {print_Zahl( i*2, 2, 1, ((i*16)+39), 151, txt_color, no_bg);} // 24 Uhr
				                                                              plot_dot(i*16+xleft, 142, txt_color);
                                                                            }
                                                                break;
													            case 4: // Wochenchart (7*24pix = 168 breit)
													                    xleft =55;
                                                                        for (uint8_t i=xleft;i<222;i+= 2) {plot_dot(i,139,txt_color);} // feine Skala  ( 2h)
                                                                        for (uint8_t i=xleft;i<222;i+=24) {plot_dot(i,140,txt_color); plot_dot(i,141,txt_color);} // Ziffer Skala (24h)
                                                                        print_Str("Mo", 1, 56,151,txt_color,no_bg);
			                                                            print_Str("Di", 1, 80,151,txt_color,no_bg);
			                                                            print_Str("Mi", 1,104,151,txt_color,no_bg);
			                                                            print_Str("Do", 1,128,151,txt_color,no_bg);
			                                                            print_Str("Fr", 1,152,151,txt_color,no_bg);
			                                                            print_Str("Sa", 1,176,151,txt_color,no_bg);
			                                                            print_Str("So", 1,200,151,txt_color,no_bg); 
                                                                break;
													            case 5: // Monatschart
													                    xleft =45;
														                for (uint8_t i= xleft;i<231;i+= 3) {plot_dot(i,139,txt_color);} // feine Skala  (12h)
				                                        	            for (uint8_t i= xleft;i<106;i+=12) {plot_dot(i,140,txt_color);} // Ziffer Skala (2 Tage)  1. bis  9.
                                                        	            for (uint8_t i=106;i<231;i+=18)    {plot_dot(i,140,txt_color);} // Ziffer Skala (3 Tage) 11. bis 31.
                                                        	            for (uint8_t i=  1;i< 10;i+= 2) 
																		    {print_Zahl( i, 1, 1, (42+(i*6)), 151, txt_color, no_bg); plot_dot(45+(i*6),150,txt_color);} // 1.3,4.6,8
				                                        	            for (uint8_t i=11;i< 31;i+= 3)
																		    {print_Zahl( i, 2, 1, (39+(i*6)), 151, txt_color, no_bg); plot_dot(48+(i*6),150,txt_color);} // 10,13,16,19,22,25,28,31
                                                                break;
													            case 6: // Jahreschart
													  	                xleft =45;
													                    for (uint8_t i=xleft;i<235;i+= 4) {plot_dot(i,139,txt_color);} // feine Skala (1 Woche)
                                                                        for (uint8_t i=xleft;i<235;i+=16) {plot_dot(i,140,txt_color); plot_dot(i,141,txt_color);} // grobe Skala (1 Monat)
                                                                        print_Str("J", 1, 46,151,txt_color,no_bg);
			                                                            print_Str("F", 1, 62,151,txt_color,no_bg);
			                                                            print_Str("M", 1, 78,151,txt_color,no_bg);
			                                                            print_Str("A", 1, 94,151,txt_color,no_bg);
			                                                            print_Str("M", 1,110,151,txt_color,no_bg);
			                                                            print_Str("J", 1,126,151,txt_color,no_bg);
			                                                            print_Str("J", 1,142,151,txt_color,no_bg);
			                                                            print_Str("A", 1,158,151,txt_color,no_bg);
 			                                                            print_Str("S", 1,174,151,txt_color,no_bg);
			                                                            print_Str("O", 1,190,151,txt_color,no_bg);
			                                                            print_Str("N", 1,206,151,txt_color,no_bg);
			                                                            print_Str("D", 1,222,151,txt_color,no_bg);
                                                                        print_Str("20",1,231,151,txt_color,no_bg);
                                                                break;
												              }
                                                        Line(xleft-1,37,xleft-1,139,1,txt_color);
    		                                            // Skala (Y)Wertachse erstellen 100pixel hoch, Y start=34 ende=134
														uint8_t nullpunkt = RAM_Data.AnzOp3 * 5; // Op1 Start 0=0%, 1=5% 2=10% 3=15% 4=20% 5=25% 6=30% 7=35% 8=40% 9=45% 10=50% 11=55% 12=60% 13=65% 14=70% 15=75%
		                                                if  (RAM_Data.SenTyp==5) {nullpunkt = RAM_Data.AnzOp3 * 10;} // nullpunkt 0=0C 1=10C 2=20C 3=30C 4=40C 5=50C 6=60C 7=70C 8=80C 9=90C 10=100C 11=110C 12=-10C 13=-20C 14=-30C 15=-40C
                                                        
														uint8_t aufloesung = 1;
	                                                    switch(RAM_Data.AnzOp2) // auflsung 0=10 1=20 2=50 3=100 Einheiten()C, (%)Lf, (l)Fllstand
	  		                                                  {
				                                                case 1: aufloesung = 2; break;
				                                                case 2: aufloesung = 5; break;
				                                                case 3: aufloesung =10; break;
                                                              }
                                                        
														uint8_t bar, baz;
                                                        if  (nullpunkt >110) // nur bei Celsius Minuswerten
				                                            {
				                                              bar = ((((nullpunkt/10) % 10)-1) *10); // ergibt 10(-10C) 20(-20C) 30(-30C) 40(-40C)
					                                          baz = 0; // 0 negative werte, 1 positive werte
                                                              // minuswert in skala eintragen, ohne 0
					                                          for (uint8_t i=0; i<11; i++)
					                                              { // negative Zahlen
					 	                                            if  (baz ==0) {print_Str("-",   1,xleft- 23,141-(i*10),txt_color,no_bg);} // minus vorzeichen
                                                                    if  (bar <10) {print_Zahl(bar,1,1,xleft- 11,141-(i*10),txt_color,no_bg);} //   -1 er
					                                                else          {print_Zahl(bar,2,1,xleft- 17,141-(i*10),txt_color,no_bg);} //   -10 er
														            plot_dot(xleft-2, 137-(i*10), txt_color);
														            plot_dot(xleft-3, 137-(i*10), txt_color);
								  
								                                    if  (baz ==1) {bar = bar +aufloesung;} // postivie Zahlen
						                                            if  (baz==0 && bar -aufloesung==0) {baz =1; bar=0;} // bei erreichen von 0, weiter mit positiven zahlen baz=1
						                                            if  (baz==0 && bar -aufloesung >0) {bar =bar -aufloesung;} // minuswerte subtrahieren
                                                                  }
                                                            }
                                                        else{ // skala ohne minuswerte eintragen
				                                              bar = nullpunkt;
				  	                                          for (uint8_t i=0; i<11; i++) // 10 Werte eintragen
					                                              {
                                                                    if  (bar <10)       {print_Zahl(bar,1,1,xleft- 11,141-(i*10),txt_color,no_bg);} //   1 er
					 	                                            else{if  (bar <100) {print_Zahl(bar,2,1,xleft- 17,141-(i*10),txt_color,no_bg);} //  10 er
						                                                 else           {print_Zahl(bar,3,1,xleft- 23,141-(i*10),txt_color,no_bg);} // 100 er
                                                                        }
                                                                    plot_dot(xleft-2, 137-(i*10), txt_color);
                                                                    bar = bar + aufloesung;
                                                                  }
                                                            }

						                                // Werte von MMC laden und ins chart zeichnen
                                                        if  (!Menu.MMC_Err)
			                                                { 
												              mach_dateiname(1,RAM_Data.SenTyp,RAM_Data.SenNum,RAM_Data.AnzTyp); // Datei Name zum laden erzeugen
				                                              load = mmc_log(1,RAM_Data.SenTyp,RAM_Data.SenNum,RAM_Data.AnzTyp); // Daten laden
					                                          if  (!load)
													              { if  (DEBUG) {transmitString_F (PSTR("\n\r mmc load error ..."));} // Fehler beim laden
														            print_Str(" ! MMCard Lade Fehler ! ", 2,24,93,Menu.Farbe.TextRot,no_bg);
                                                                  }
                                                              else{chart_minmax();} // Chart Kopfzeile
                                                            }
                                                      } else{ print_Str("! keine Daten gefunden !",2, 15, 93, Menu.Farbe.TextRot,no_bg);}
                                                } else{ print_Str("! kein Sensor gefunden !",2,15,93,Menu.Farbe.TextRot,no_bg);}
                                          }
						              else{ // --- Listen --- (sentyp<10)
							
								            Menu.bit6_1 =0; // ListZelle
                                            load = mmc_log(1, RAM_Data.SenTyp, RAM_Data.SenNum, RAM_Data.AnzTyp); // Daten laden
					                        if  (load==0 || load==2)
								                {
								  	              print_Str("< keine Daten auf MMC! >",2, 24, 93, Menu.Farbe.TextRot, no_bg);
									              if  (DEBUG) {transmitString_F (PSTR("\n\r mmc load error ..."));} // Fehler beim laden
                                                }
								            else{ list_draw();} // Liste zeichnen
                             	          }
                                    }
				              }
			              // [Clock]
                          else{ Rectan(2,25,237,184,0,fld_color, 1); // groes Feld lschen, inkl. Sentyp Eingabefeld Sennum Btn
				                switch(RAM_Data.AnzTyp)
					                  {
							            case 7: // digi Clock
								                print_Str("Uhr",1,215,78,txt_color,no_bg);
                                                update.big_clock =1; // Uhr muss aktualisiert werden
							            break;
                                        case 8: // analog Clock
			                                    Circle(119,104,78,1,txt_clr(RAM_Data.AnzOp2),0); // Uhr Rand
				                                print_Str("12" , 1, WatchDots[0] [4]-6, WatchDots[0] [5]+4, txt_color, no_bg); // 12
					                            print_Str( "1" , 1, WatchDots[5] [4]-3, WatchDots[5] [5]+4, txt_color, no_bg); //  1
                                                print_Str( "2" , 1, WatchDots[10][4]-3, WatchDots[10][5]+4, txt_color, no_bg); //  2
					                            print_Str( "3" , 1, WatchDots[15][4]-3, WatchDots[15][5]+4, txt_color, no_bg); //  3
					                            print_Str( "4" , 1, WatchDots[20][4]-3, WatchDots[20][5]+4, txt_color, no_bg); //  4
					                            print_Str( "5" , 1, WatchDots[25][4]-3, WatchDots[25][5]+4, txt_color, no_bg); //  5
					                            print_Str( "6" , 1, WatchDots[30][4]-3, WatchDots[30][5]+4, txt_color, no_bg); //  6
					                            print_Str( "7" , 1, WatchDots[35][4]-3, WatchDots[35][5]+4, txt_color, no_bg); //  7
					                            print_Str( "8" , 1, WatchDots[40][4]-3, WatchDots[40][5]+4, txt_color, no_bg); //  8
					                            print_Str( "9" , 1, WatchDots[45][4]-3, WatchDots[45][5]+4, txt_color, no_bg); //  9
					                            print_Str("10" , 1, WatchDots[50][4]-4, WatchDots[50][5]+4, txt_color, no_bg); // 10
					                            print_Str("11" , 1, WatchDots[55][4]-4, WatchDots[55][5]+4, txt_color, no_bg); // 11
                                                for (uint8_t i=0; i<59; i++)
					    	                        {
							                          Line(pgm_read_byte(&(WatchDots[i][0])), pgm_read_byte(&(WatchDots[i][1])), // x1 y1
							                               pgm_read_byte(&(WatchDots[i][2])), pgm_read_byte(&(WatchDots[i][3])), // x2 y2
							                               1,txt_color); // font, farbe
                                                    }
                                                Line(119,108,pgm_read_byte(&(WatchDots[mm][8])),pgm_read_byte(&(WatchDots[mm][9])),2,txt_color); // Minutenzeiger erstellen
					                            hh_foo =hh; if  (hh_foo >11) {hh_foo =hh_foo-12;};
					                            Line(119,108,pgm_read_byte(&(WatchDots[((hh_foo*5)+(mm/12))][10])),pgm_read_byte(&(WatchDots[((hh_foo*5)+(mm/12))][11])),5,txt_color); // Stundenzeiger erstellen
				                        break;
				                        case 9: // --- Video Kanal erstellen (*optional*) ---
                                        break;  //
                                      }
                              }
                        }
			break;
			case 2: // --- Refresh ---
		            if  (RAM_Data.AnzTyp <3)
		                { 
						  // [Digital Felder] //  RAM_Data.Feld[] 0-8 = 9D. 9-12 =4D, 13=1Digital Feld
						  switch(RAM_Data.AnzTyp)
		                        { //      start = 0; felder = 9;        // --- 9 Felder ---
				                  case 1: start = 9; felder = 4; break; // --- 4 Felder ---
				                  case 2: start =13; felder = 1; break; // --- 1 Feld   ---
                                }
                          for (uint8_t i =start; i <start + felder; i++) // Alle digit.Felder durchsuchen
	                          { 
		                        if  (RAM_Data.Feld[i].SenTyp) // Feld aktiv?
				                    { 
					                  // xleft und ytop einstellen
				                      switch(RAM_Data.AnzTyp)
					                        {
							                  case 0: // 9 Ausgabe Felder
							                          if  (i==1 || i==4 || i==7) {xleft =  6;}
 				                                      if  (i==2 || i==5 || i==8) {xleft = 85;} 
					                                  if  (i==3 || i==6 || i==9) {xleft =165;}
                                                   
												      if  (i<3)                          {ytop =  25; height = 40;} // 9F 1.reihe
		       		                                  else{ if  (i<6)                    {ytop =  68; height = 40;} // 9F 2.reihe
			        	                                    else{ if  (i<9)              {ytop = 111; height = 40;} // 9F 3.reihe
						                                          else{ if  (i<11)       {ytop =  25; height = 61;} // 4F 1.reihe
			              	                                            else{ if  (i<13) {ytop =  89; height = 61;} // 4F 2.reihe
				               	                                              else       {ytop =  25; height =126;} // 1F 
                                                                            }
                                                                      }
                                                                }
                                                          }

                                                      if  (i <4)
													      {ytop = 55;}
                                                      else{ if  (i <7)
													            {ytop = 98;}
                                                            else{ if (i <10)
															         {ytop = 141;}
		                                                        };
						                                  };
                                                      font =2;
                                              break;
							                  case 1: // 4 Ausgabe Felder
				                                      if  (i==10 || i==12) {xleft = 15;}
				                                      if  (i==11 || i==13) {xleft =134;}
               
                                                      if  (i <12)
	 		                                              {ytop = 65;}
                                                      else{ytop =129;}
									                  font =2;
                                              break;
							                  case 2: // 1 Ausgabe Feld
							                          xleft = 6;
									                  ytop  = 135;
									                  font =4;
							                  break;
                                            }
			 		                  
									  // Farben einstellen
	  	                              color_set(RAM_Data.Feld[i].AnzOp1);
                                      fld_color = Menu.Farbe.Dunkel+(5*Menu.Farbe.Sub);
                                      txt_color = txt_clr(RAM_Data.Feld[i].AnzOp2);
							          
									  // Wert ausgeben
			                          if  (Menu.Value[array_zu_sentyp(2, RAM_Data.Feld[i].SenTyp, RAM_Data.Feld[i].SenNum)] > -32000)
									      { // gltiger Sensorwert vorhanden
                                            label_value(6,RAM_Data.Feld[i].SenTyp,RAM_Data.Feld[i].SenNum,font,xleft,ytop,txt_color,gnd_color); // Value aus Puffer "Value[]" unten ausgeben (Messdaten)
										  }
						              else{print_Str("SensFail",font,xleft+11,ytop,txt_color,no_bg);} // Fehler
                                    } // Feld Aktiv Ende
							  } // for() ende
                        } // digital felder ende
         
		            // [[[digital Clock]]]
                    else{ if  (RAM_Data.AnzTyp ==7)
                              { // Uhrzeit
			                    display_time(1);	
                              }
						   // [[[analog Clock]]]
                          else{ if  (RAM_Data.AnzTyp ==8)
		                            { 
									  uint8_t hh_foo;
				                      uint8_t hh_bar, ss_foo, ss_bar, mm_foo, mm_bar;
			                          if  (!Menu.Bclocked)
				                          {
					                        Circle(119,104,65,0,fld_color,1); // alte Uhrzeiger lschen
					                        print_Str("10" , 1, WatchDots[50][4]-4, WatchDots[50][5]+4, txt_color, fld_color); // 10 neu zeichnen
					                        print_Str("11" , 1, WatchDots[55][4]-4, WatchDots[55][5]+4, txt_color, fld_color); // 11 "
					                        Menu.Bclocked =1;
                                          }
			                          ss_foo =ss;
				                      if  (ss_foo ==0) {ss_bar=59;} else{ss_bar =ss_foo -1;}
				                      mm_foo =mm;
				                      if  (mm_foo ==0) {mm_bar=59;} else{mm_bar =mm_foo -1;}
				                      if  (hh >11) {hh_foo =(((hh-12)*5) + (mm_foo/12));}
                                      else         {hh_foo =((hh*5)+(mm_foo/12));}
				                      if  (hh_foo ==0) {hh_bar=59;} else{hh_bar =hh_foo -1;}
		      
			                          Line(119,108,pgm_read_byte(&(WatchDots[hh_bar][10])),pgm_read_byte(&(WatchDots[hh_bar][11])),5,fld_color); // alten Stundenzeiger lschen
			                          Line(119,108,pgm_read_byte(&(WatchDots[mm_bar][8])),pgm_read_byte(&(WatchDots[mm_bar][9])),2,fld_color); // alten Minutenzeiger lschen
			                          Line(119,108,pgm_read_byte(&(WatchDots[ss_bar][6])),pgm_read_byte(&(WatchDots[ss_bar][7])),1,fld_color); // alten Sekundenzeiger lschen
			                          Line(119,108,pgm_read_byte(&(WatchDots[hh_foo][10])),pgm_read_byte(&(WatchDots[hh_foo][11])),5,txt_color); // Stundenzeiger erstellen
			                          Line(119,108,pgm_read_byte(&(WatchDots[mm_foo][8])),pgm_read_byte(&(WatchDots[mm_foo][9])),2,txt_color); // Minutenzeiger erstellen
			                          Line(119,108,pgm_read_byte(&(WatchDots[ss_foo][6])),pgm_read_byte(&(WatchDots[ss_foo][7])),1,txt_color); // Sekundenzeiger erstellen
			                        }
                       
                                // [[[Video Kanal]]]
                                else{ if  (RAM_Data.AnzTyp ==9)
			                              {
			                                ;
			                              }
                                    }
                              }
                        }
            break;
          }
  }



/****************************************************************************
 Funktion Auswahl Button

 - press: 0 = Button build
          1 = Button links gedrckt
		  2 = Button rechts gedrckt

 - tab im Hauptmen;
          12 = Sensor Typ  Btn (Menu.SenTyp)  / RAM_Data.Feld[15].SenTyp 0-12
          13 = Sensor Nr.  Btn (Menu.SenNum)  / RAM_Data.Feld[15].SenNum 0-15
	      14 = Anzeige Typ Btn (Menu.AnzTyp)  / RAM_Data.Anzeige.AnzTyp  0- 9
	      15 = Anzeige Option1 (Menu.AnzOp1)  / RAM_Data.Anzeige.AnzOp1  0-15
	      16 = Anzeige Option2 (Menu.AnzOp2)  / RAM_Data.Anzeige.AnzOp2  0-15
       im Pumpenmen;
          75 = DosProgramm Btn (Menu.4bit_1) 0- 3
	      77 = Bews.Woche Btn (Menu.4bit_2) 0-15
       im Relaismen;
           3 = B.Pr/Relais Btn (Menu.6bit_1) 0-31
	       4 = Aktor Modus Btn (Menu.2bit_1) 0- 3
	       5 = Woche/P.Del Btn (Menu.4bit_1) 0-15 ***"bei P.Delay Option fr Handstart dazu***
          12 = Sensor Typ  Btn (Menu.4bit_2) 0-31
	      13 = Sensor Num  Btn (Menu.4bit_3) 0-15
	      14 = Relais Beschriften (Menu.1bit_1)
	      15 = Sensor Beschriften (Menu.1bit_2)
	      16 = UND ODER Btn (Menu.2bit_2)
	      18 = (<) (>)  Btn (Menu.2bit_3)
       im Kalendermen;
          53 = "j Jahr" Btn (Menu.1bit_2)
	      54 = "Farbauswahl (Menu.4bit_1)
	      55 = Uhrzeit stel (Menu.1bit_3)
	      57 = Datum stellen(Menu.1bit_4)
       im Info/Setup Men;
           2 = LeistenFarbe (Menu.4bit_1  / RAM_Data.Farbe.Leiste)
           3 = Button Farbe (Menu.4bit_2  / RAM_Data.Farbe.Button)
	       4 = Screen Farbe (Menu.1bit_1  / RAM_Data.Farbe.Grund)
	       5 = DCF Ein/Aus  (Menu.1bit_2  / RAM_Data.DCF)
	       6 = MMC_Clear   
	       7 = ScreenSaver  (Menu.1bit_3  / RAM_Data.ScS)

		  Die Variablen werden nach verlassen des Mens in den RAM gespeichert!
 ****************************************************************************/
void auswahl_btn(uint8_t press, uint8_t tab)
  {
   uint16_t ytop  =0;
   uint8_t xleft  =0;
   uint8_t breite =0;
   uint8_t wert   =0;
   uint16_t grenze =0;
   switch (Menu.Window)
          { case Haupt_Menu :   switch(tab)
			                          { case 12: xleft=  2; ytop=154; breite= 78; grenze=12; break; // SenTyp
									    case 13: xleft=161; ytop=154; breite= 78; grenze=sens_anzahl(RAM_Data.SenTyp); break; // SenNum
									    case 14: xleft=  2; ytop=187; breite= 78; grenze= 9; break; // AnzTyp
									    case 15: xleft= 82; ytop=187; breite= 77; grenze=opt1_anzahl(RAM_Data.AnzTyp); break; // AnzOp1
									    case 16: xleft=161; ytop=187; breite= 78; grenze=opt2_anzahl(RAM_Data.AnzTyp); break; // AnzOp2
                                      }
		    break;
		    case Pumpen_Menu:   switch(tab)
			                          { case 75: xleft=  2; ytop=220; breite= 78; grenze= 3; break; // B-Programm
									    case 77: xleft=161; ytop=220; breite= 78; grenze=15; break; // B-Woche
                                      }
            break;
            case Relais_Menu:   switch(tab)
			                          { case  3: xleft=  2; ytop= 25; breite= 78; grenze=31; break; // Aktor
									    case  4: xleft= 82; ytop= 25; breite= 77; grenze= 2; break; // Modus
									    case  5: xleft=161; ytop= 25; breite= 78; grenze=15; break; // P-Delay/B-Woche
									    case 12: xleft=  2; ytop=154; breite= 78; grenze=12; break; // SenTyp
									    case 13: xleft= 82; ytop=154; breite= 77; grenze=sens_anzahl(RAM_Data.SenTyp); break; // SenNum
									    case 14: xleft=161; ytop=154; breite= 78; grenze= 2; break; // UND ODER
									    case 15: xleft=  2; ytop=187; breite=118; grenze= 1; break; // Aktor Label
									    case 16: xleft=121; ytop=187; breite=118; grenze= 1; break; // Sensor Label
									    case 17: xleft=  2; ytop=220; breite= 78; grenze= 1; break; // Minus
										case 19: xleft=161; ytop=220; breite= 78; grenze= 2; break; // Kleiner Grer
                                      }
            break;
			case Relais_Liste:  switch(tab)
			                        {
									 ;
									}
            break;
			case Kalender_Menu: switch(tab)
			                          { case 53: xleft=  2; ytop=187; breite= 78; grenze= 1; break; // "jedes Jahr"
									    case 54: xleft=161; ytop=187; breite= 78; grenze=15; break; // Farbauswahl
										case 55: xleft=  2; ytop=220; breite= 78; grenze= 1; break; // Uhrzeit stellen
										case 57: xleft=161; ytop=220; breite= 78; grenze= 1; break; // Datum stellen
                                      }
            break;
			case Info_Menu:     switch(tab)
			                          { case  2: xleft=  2; ytop= 25; breite= 78; grenze=15; break; // Menleisten Farbe
										case  3: xleft= 82; ytop= 25; breite= 77; grenze=15; break; // Btn Farbe
										case  4: xleft=161; ytop= 25; breite= 78; grenze= 1; break; // Screen Farbe
										case  5: xleft=  2; ytop= 58; breite= 78; grenze= 1; break; // DCF ON/OFF
										case  6: xleft= 82; ytop= 58; breite= 77; if(press){return;} break; // MMC Clear???
										case  7: xleft=161; ytop= 58; breite= 78; grenze= 1; break; // Screen Saver
									  }
            break;
          }

   // --- Button Pressed ---
   if  (press)
       { // Btn Wert lesen
	     wert = btn_wert( 0, tab);
         if  (press==2) { if  (wert <grenze) {wert ++;} else{wert =0;     }} // rechts gedrckt, wert ++
		 else           { if  (wert >0)      {wert --;} else{wert =grenze;}} // links gedrckt, wert --
         // neuen Btn Wert setzen
	     btn_wert(wert, tab);

		 // --- andere Variablen die sich durch Btn Wert nderung mit ndern(rckstellen) ---
		 if  (tab==12 || (Menu.Window==Relais_Menu && tab==3)) {RAM_Data.SenNum=0;}

         if  (Menu.Window==Haupt_Menu && (tab==12 || tab==13 ||tab==14)) // SensorTyp, SensorNum, AnzeigeTyp gendert ...
		     { 
			   if  (RAM_Data.AnzTyp>2 && RAM_Data.AnzTyp<7) // Charts und Listen Variablen zurcksetzen
			       { if  ( RAM_Data.SenTyp<10)    // Charts
					     { Menu.bit16_1 = -32000; // Min-Wert
	                       Menu.bit16_2 = -32000; // Max-Wert
	               		 }                   // Listen
                     else{ Menu.bit16_1 = 0; // Listen Seite (ausgewhlte Listen Seite)
					       Menu.bit16_2 = 0; // Seitenanzahl (max Anzahl der mglichen Listen Seiten)
						   Menu.bit6_1  = 0; // List Zelle , zuletzt beschriebene Zelle auf der Listen Seite
                         }
			   		 RAM_Data.AnzOp1 =0;
	                 RAM_Data.AnzOp2 =3;
		           }
		       if  (tab==12)
			       { if  (RAM_Data.SenTyp==1 || RAM_Data.SenTyp==2) // SensorTyp Wahl ohne Zeitperiode/Zeitpunkt im Hauptmen
				         { if  (press==2) {RAM_Data.SenTyp =3;} // Sprung zu "TagesChart"
				           else           {RAM_Data.SenTyp =0;} // Sprung zu "Aus" zum lschen eines Feldes
				         }
                   }
               if  (tab==14)
			       { // Anzeige Typ Btn
			         if  ((press==2 && RAM_Data.AnzTyp==0) || (press==1 && RAM_Data.AnzTyp==6))
			             { // SenTyp/Nr Btn und eingabefeld erstellen bei switch von video auf 9digi(9) oder von clock auf chart/liste(6)
		                   color_set(Hintergrund);
						   Line(  0,152,239,152,2,Menu.Farbe.Grund);
		                   Line( 79,152, 79,186,2,Menu.Farbe.Grund);
		                   Line(159,152,159,186,2,Menu.Farbe.Grund);
                           eingabe_fld();
                           button_build(161,ytop,77,31,RAM_Data.Farbe.Button); // SenNum Btn neu erstellen
						   button_text(13,161,ytop); // SenNum Btn beschriften
                         }
				   }
             }
       } // press ende
 
   // --- Button Build ---
   button_build(xleft,ytop,breite,31,RAM_Data.Farbe.Button); // SenNum Btn neu erstellen
   button_text(tab,xleft,ytop); // SenNum Btn beschriften
  };



/*************************************************
 gibt die Y_top Position des Eingabe Feldes zurck
 *************************************************/
uint16_t ytop_eingabe_fld(void)
  {
    switch(Menu.Window)
          {
	 	    case Haupt_Menu:  return 154; break;
		    case Pumpen_Menu: return 254; break;
            case Relais_Menu: return 253; break;
          }
    return 0;
  }



/********************************************
 Funktion Relais Feld;  235breit, 62 hoch
 ********************************************/
void relais_btn(void)
  {
   uint8_t x_foo; // x-Pos.
   uint16_t y_foo; // y-Pos
   color_set(RAM_Data.Farbe.Button);

   // --- Button build ---
       { 
	     button_build(2,220,236,61,RAM_Data.Farbe.Button);
	     // 16 Relais Kstchen erstellen
		 Line(8,225,232,225,1,Weiss6);
		 Line(8,251,232,251,1,Weiss6);
		 Line(8,278,232,278,1,Weiss6);
		 for (uint8_t i=0; i<9; i++) {x_foo =i*28+  8; Line(x_foo,222,x_foo,274,1,Weiss6);}
	    
		 for (uint8_t i=0; i <16; i++)
             { // Relaiszustand, Nr. & Modus darstellen
			   if  (i <8) {x_foo =i; y_foo=226;} 
			   else       {x_foo =i-8; y_foo=252;}
			   x_foo= x_foo * 28 +13;
			   
			   if  (i <9) {print_Zahl(i+1,1,1,x_foo +6, y_foo +16,Menu.Farbe.Text,no_bg);} // Relais Nr mit 1 Ziffer
               else       {print_Zahl(i+1,2,1,x_foo +3, y_foo +16,Menu.Farbe.Text,no_bg);} // Relais Nr mit 2 Ziffern
   	           
			   switch(RAM_Data.Aktor[i].Modus)
			         {
					   case 0: print_Str("OFF" ,1,x_foo,  y_foo+25,Menu.Farbe.Text,no_bg); break; // Relais Sschaltmodus Off
		               case 1: print_Str("AUTO",1,x_foo-3,y_foo+25,Menu.Farbe.Text,no_bg); break; // "      "            Auto
 		               case 2: print_Str("ON"  ,1,x_foo+3,y_foo+25,Menu.Farbe.Text,no_bg); break; // "      "            On
                     }

   	           if  (RAM_Data.Aktor[i].Status) {Rectan(x_foo,y_foo+3,x_foo+18,y_foo+6,0,Menu.Farbe.TextGruen,1);} // Relais Zustand On
		       else                           {Rectan(x_foo,y_foo+3,x_foo+18,y_foo+6,0,Menu.Farbe.TextRot,  1);} // Relais zustand Off
			 }
	   }
  }



/***************************
 Funktion Info Feld Button;
 Parameter:
 - action: 0 Btn build
           1 Btn refresh
 ***************************/
void info_fld(uint8_t press)
  {
   color_set(RAM_Data.Farbe.Leiste);

   // Button build
   if  (!press) {Rectan(2,283,118,317,0,Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1);}

   if  (!Menu.MMC_Err) // SD-MMC status
	   {Rectan(4,303,44,315,0,Menu.Farbe.TextGruen,1);} //green ok
   else{Rectan(4,303,44,315,0,Menu.Farbe.TextRot,1);} // red error

   for (uint8_t i=4; i<8; i++) // SHT-Sensoren Hardware check
       {
	     if  (Menu.Value[i] > -32000)
		     {Rectan((i*12)+69,303,(i*12)+81,315,0,Menu.Farbe.TextGruen,1);} // grn OK
         else{Rectan((i*12)+69,303,(i*12)+81,315,0,Menu.Farbe.TextRot,1);} // rot Fail
       }
   print_Str("MMC/SD SHT 1 2 3 4",1,6,313,Menu.Farbe.Text,no_bg);
  }



/**************************************************
 Funktion Uhr/ Kalender Button; minimale breite=118
 Parameter:
 - action : 0 = Feld build
            1 = Feld refresh
 **************************************************/
void uhr_fld(uint8_t press)
  {
   color_set(RAM_Data.Farbe.Leiste);
   uint16_t color =Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub);
   // build
   if  (!press)
       {
	     Rectan(121,283,237,317,0,color,1);
         print_Str("Uhr",1,149,297,Menu.Farbe.Text,color); 
	   }

    // refresh
   if  (press)
       { // beschriften
		 display_time(0);
  	   }
  }



/*************************************************
 Funktion Einschaltbedingungsfeld
 Parameter:
 - action : 0 = 6 Felds build
            1 = Feld pressed, WertEingabe starten
 *************************************************/
void switch_fld(uint8_t press)
  {
   uint8_t  width  =  76;
   uint8_t  height =  30;
   uint8_t  xleft  =   2;

   uint16_t color;
   uint16_t ytop;

   // --- Button pressed ---
   if  (press)
       {
		 if  (!Menu.bit1_1 && !Menu.bit1_2)
	         { // kann ein Wert ins Switch-Feld eingetragen werden?
	           if  ((Menu.Tab== 6 || Menu.Tab== 9) || // Feld 1/ 4
			        (Menu.Tab== 7 && RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[0].SenTyp) || // Feld 2
			        (Menu.Tab== 8 && RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[0].SenTyp &&  // Feld 3
					                 RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[1].SenTyp) || // "
			        (Menu.Tab==10 && RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[3].SenTyp) || // Feld 5
		            (Menu.Tab==11 && RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[3].SenTyp &&  // Feld 6
			                         RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[4].SenTyp))   // "
		           { // X Y des SwitchFeld
	                 if  (Menu.Tab==7 || Menu.Tab==10) {xleft = 81; width =77;}
		             if  (Menu.Tab==8 || Menu.Tab==11) {xleft =161;}
                     if  (Menu.Tab<9) {ytop =62; color_set(Gruen);} else{ytop =95; color_set(Rot);}
					 // Feld Farbe
					 color = Menu.Farbe.Dunkel +(3 * Menu.Farbe.Sub);
                     // SwitchFeld erstellen
					 Rectan(xleft,ytop,xleft+width,ytop+height,0,color,1);
                     // Einstellungen bernehmen
					 RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6].SenTyp = Menu.bit4_2; // SensorTyp
					 if  (Menu.bit4_2 >0) // weitere Einstellungen bernehmen, nur wenn ein SensorTyp eingestellt ist...
		                 { RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6].SenNum = Menu.bit4_3; // SensorNum
		                   RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6].Aor = Menu.bit2_2; // & /
                           RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6].Grk = Menu.bit2_3; // < >
		                   RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6].Neg = Menu.bit1_3; // -
		                   for (int8_t i=0; i<4; i++)
		                       {RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6].Value[i]=Menu.Ascii[i];} // Value aus Ascii[0] bis [3]
		                 }
                     // im EEPROM speichern
                     cli();
	                 eeprom_write_block(&RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6],
	                                    &EEP_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6],
	                             sizeof (RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[Menu.Tab -6]));
                     sei();
			         // Sensor Name ins Feld scheiben oben
                     label_value(0, RAM_Data.SenTyp,RAM_Data.SenNum,1,xleft,ytop+13, Menu.Farbe.Text,no_bg);
                     // Wert ins SwitchFeld schreiben unten
				     label_value(7, Menu.bit6_1, Menu.Tab-6, 1, xleft+2, ytop+25, Menu.Farbe.Text, no_bg); // Value aus der "Schaltbedingung" augeben
					 eingabe_fld();
                     return;
                   }
		     }
       }

   // --- 6 SwitchFelder build? ---
   else{
	     for (uint8_t i=0; i<6; i++)
	         {
		 	   // XY & Farbe einstellen
	           if  (i==1 || i==4) {xleft = 81; width =77;}
                else              {xleft =161;}
			   if  (i<3) {ytop =62; color_set(Gruen);} else{ytop =95; color_set(Rot);}
               // Feld Farbe
			   color = Menu.Farbe.Dunkel +(3 * Menu.Farbe.Sub);
			   // Feld build
	    	   Rectan(xleft,ytop,xleft+width,ytop+height,0,color,1);
             
			   // Feld beschriften
			   if  (RAM_Data.Aktor[Menu.bit6_1].Schaltbedingung[i].SenTyp)
			       { 
    			     // Sensor Name ins Feld scheiben oben
                     label_value(0, RAM_Data.SenTyp,RAM_Data.SenNum,1,xleft,ytop+13, Menu.Farbe.Text,no_bg);
                     // Wert ins SwitchFeld schreiben unten
				     label_value(7, Menu.bit6_1, Menu.Tab-6, 1, xleft+2, ytop+25, Menu.Farbe.Text, no_bg); // Value aus der "Schaltbedingung" augeben
                   }
             }
	   }
  }



/****************************************************************************
 Dosiertabelle
 Parameter[in]
 - press:0 = Tabelle zum eingestellten Programm laden und erstellen
         1 = eine ganze Zeile oder eine Wert-Zelle links gedrckt
		 2 = "                                     rechts gedrckt
		 3 = Wochenwechsel daten aus der MenStruktur nehmen und alle Wert-Zellen aktualisieren
		 4 = Progr.wechsel daten aus dem eeprom laden nehmen und alle Wert-Zellen aktualisieren
		 5 = Enter Taste gedrckt
		 6 = CLR Taste gedrckt

                                         (Y)   26
   Farbe    Mo  Di  Mi  Do  Fr  Sa  So    = 41
 ----------+---+---+---+---+---+---+---+ <---  48
  MagnV.(l)|010|010|010|010|010|015|015|  = 62
 ----------+---+---+---+---+---+---+---+ <---  67
  DosP1(ml)|   |   |   |   |   |   |   |  = 81
 ----------+---+---+---+---+---+---+---+ <---  86
  DosP2(ml)|   |   |   |   |   |   |   |  =100
 ----------+---+---+---+---+---+---+---+ <--- 105
  DosP3(ml)|   |   |   |   |   |   |   |  =119
 ----------+---+---+---+---+---+---+---+ <--- 124
  DosP4(ml)|004|004|004|004|004|006|006|  =138
 ----------+---+---+---+---+---+---+---+ <--- 143
  DosP5(ml)|025|025|025|025|025|025|   |  =157
 ----------+---+---+---+---+---+---+---+ <--- 162
  DosP6(ml)|020|   |   |   |   |   |   |  =176
 ----------+---+---+---+---+---+---+---+ <--- 181
  DosP7(ml)|   |   |   |   |   |   |030|  =195
 ----------+---+---+---+---+---+---+---+ <--- 200
  DosP8(ml)|Xxx|   |   |   |   |   |   |  =214
 ----------+---+---+---+---+---+---+---+ <--- 219
           ^67 ^92 ^117^142^167^192^217^
            63  88 113 138 163 188 213 
 ****************************************************************************/
void dosier_tabelle(uint8_t press)
  {
    uint8_t wert, x_foo, spalte, zeile, mmc_ok;
	uint16_t y_foo;
    color_set(Hintergrund);
	uint16_t color =Menu.Farbe.Text;
	uint16_t bg_clr =Menu.Farbe.Grund;

	if  (!press)
	    { // Tabellenkopf erstellen ...
		  print_Str("Mo"       ,1, 67,41,color,no_bg);
          print_Str("Di"       ,1, 92,41,color,no_bg);
	      print_Str("Mi"       ,1,117,41,color,no_bg);
	      print_Str("Do"       ,1,142,41,color,no_bg);
	      print_Str("Fr"       ,1,167,41,color,no_bg);
	      print_Str("Sa"       ,1,192,41,color,no_bg);
	      print_Str("So"       ,1,217,41,color,no_bg);
        }
     
    if  (!press || press==4 || (press==6 && !Menu.bit2_1))
	    { // Dosierdaten eines Dosierprogramms von MMC in Men Struktur laden
          if  ((!Menu.MMC_Err && press <5) || press==6)
		      {
	            if  (press !=6)
				    { mmc_ok = mmc_dosierdaten(0);
					  if  (!mmc_ok)
					      { if  (DEBUG) {transmitString_F (PSTR("\n\r mmc load error !"));}
                          }
                    }
                if  (press==6 || !mmc_ok)
				    { // Tabelle lschen
					  for (uint8_t woche =0; woche <16; woche++)
					      {
						    for (zeile =0; zeile <9; zeile++)
							    {
								  for (spalte =0; spalte <7; spalte++)
								      {
					                    Menu.Tabellenwoche[woche].Zeile[zeile].Spalte[spalte] =0;
                                      }
                                }
                          }
			        }
              }
        }

    if  (!press || ((press==1 || press==2) && Menu.Tab==2))
	    { // Farbauswahl Btn ...
		  if  (press==2) { if  (Menu.bit4_1 <15) {Menu.bit4_1 +=1;} else{Menu.bit4_1 = 0;}} // Farbe switchen
		  if  (press==1) { if  (Menu.bit4_1 > 0) {Menu.bit4_1 -=1;} else{Menu.bit4_1 =15;}} // "

          color_set(Menu.bit4_1 +1);
		  Rectan(2, 26, 62, 47, 0, Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1); // erstellen
		  print_Str("Farbe", 1, 15, 41, Menu.Farbe.Text, no_bg); // & beschriften
		}
  
    if  (!press || (press ==6 && !Menu.bit2_1))
	    { // ganze Tabellenseite erstellen
          for (zeile=0; zeile<9; zeile++)
	          { 
			    color_set(RAM_Data.Farbe.Zeile[zeile] +1); // Zeilenfarbe setzen
			    y_foo =zeile*19+48; // Zeilen Y-Pos oben
			    if  (!press)
				    { // Zeile zeichnen und LabelZelle beschriften
				      Rectan(2, y_foo+1,237,y_foo+18,0,Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1);
			          label_value(0,13,zeile,1,6,y_foo+14,Menu.Farbe.Text,no_bg); // Label aus EEPROM ausgeben
                    }
                else{ Rectan(64, y_foo+1,237,y_foo+18,0,Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1);}

				for (spalte=0; spalte<7; spalte++)
				    { // Wert-Zellen einer Zeile
					  x_foo =spalte*25+63;
					  Line(x_foo, y_foo+1, x_foo, y_foo+19, 1,bg_clr); // |Spaltentrennung links
					  wert =Menu.Tabellenwoche[Menu.bit4_2].Zeile[zeile].Spalte[spalte];
					  if  (wert) { print_Zahl(wert,3,1,x_foo+4,y_foo+14,Menu.Farbe.Text,no_bg);} // Wert in Zelle eintragen
					}
              }
	    }

	if  ((press==1 || press==2) && Menu.Tab>1)
	    {
		  Menu.bit6_1 = Menu.Tab -3; // Zellen Nr speichern
		  zeile  = Menu.bit6_1 / 8; // 0 - 8
		  spalte = (Menu.bit6_1 - 1) - (zeile * 8); // 0-6
		  Menu.bit2_1 = 2 - (zeile %8); // Zellenauswahl aktivieren, 2=Label 1=Wert-Zelle aktiviert
		  if  (Menu.bit2_1==1)
		      { // LabelZelle?
		   	    label_value(3, 13,zeile, 1, 0, 0, 0, 0); // Label aus EEPROM in den Puffer laden
              }
          else{ // WertZelle?
			    wert = Menu.Tabellenwoche[Menu.bit4_2].Zeile[zeile].Spalte[spalte]; // Value 
        		Menu.Ascii[0] =wert/100; wert =wert%100; Menu.Ascii[1] =wert/10; Menu.Ascii[2] =wert%10;
              }
          Menu.Cursor=1; // cursor auf 1.pos.
		  eingabe_fld();
		  return;
	    }
    if  (press==5 || press==6)
	    { // ENTER(5)  oder CLEAR(6) um Label, Wert oder Tabellenwoche(!bit2_1) zu speichern/ lschen
		  if  (!Menu.bit2_1) 
		      { // keine Zelle ausgewhlt
				if  (press==5) {mmc_dosierdaten(1);} // ENTER? =Dosierprogramm speichern
              }
          else{
		        zeile  = Menu.bit6_1 /8;
                spalte = (Menu.bit6_1 -1) - (zeile * 8);
	            x_foo  = spalte * 25 + 63;
		        y_foo  = zeile  * 19 + 48;
		        if  (press==5)
		            { // Enter
			          if  (Menu.bit2_1 ==1) // ZellenWert speichern
                          { color_set(RAM_Data.Farbe.Zeile[zeile]+1);
                            wert =(Menu.Ascii[0]*100) +(Menu.Ascii[1]*10) +Menu.Ascii[2];
                            Menu.Tabellenwoche[Menu.bit4_2].Zeile[zeile].Spalte[spalte] = wert; // wert bernehmen
                            Rectan(x_foo, y_foo, x_foo+24, y_foo+18, 0, Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1); // Zelle zeichnen
                            if  (wert) {print_Zahl(wert, 3, 1, x_foo+3, y_foo+13, Menu.Farbe.Text, no_bg);} // Wert rein
                          }
                      else{ // zeilenLabel fr Dos-Pumpe und Zeilenfarbe speichern
				            RAM_Data.Farbe.Zeile[zeile] = Menu.bit4_1;
					        color_set(Menu.bit4_1 +1); // Zeilenfarbe
		                    Rectan(2, y_foo+1,237,y_foo+18, 0, Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1); // Zeile erstellen
		                    label_value(2, 13, zeile, 0, 0,       0, Menu.Farbe.Text, no_bg); // Label aus Puffer Ascii[] in EEPROM schreiben
                            label_value(0, 13, zeile, 1, 6,y_foo+14, Menu.Farbe.Text, no_bg); // Label aus EEPROM ausgeben
								
							for (spalte=0; spalte<7; spalte++)
		                        { // Zeile erstellen
			                      x_foo =spalte*25+63;
			                      Line(x_foo, y_foo+1, x_foo, y_foo+19, 1,bg_clr); // |Spaltentrennung links
			                      wert =Menu.Tabellenwoche[Menu.bit4_2].Zeile[zeile].Spalte[spalte];
			                      if  (wert) {print_Zahl(wert,3,1,x_foo+4,y_foo+14,Menu.Farbe.Text,no_bg);} // Wert in Zelle eintragen
			                    }
                          }
				    }
			    else{ // CLEAR
		              if  (Menu.bit2_1 ==1)
					      { // Zellen Wert lschen
						    Menu.Tabellenwoche[Menu.bit4_2].Zeile[zeile].Spalte[spalte] =0;
                            color_set(RAM_Data.Farbe.Zeile[zeile]+1);
                            Rectan(x_foo, y_foo, x_foo+24, y_foo+18, 0, Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1); // Zelle zeichnen
					      }
                      if  (Menu.bit2_1 ==2)
					      { // Zeilen Label reseten
						    label_value(4, 13, zeile, 0,0,0,0,0); // Label reseten
							label_value(0, 13, zeile, 1, 6, y_foo+14, Menu.Farbe.Text, no_bg); // Label ausgeben
						  }
                    }
		      }
		  if  (Menu.bit2_1) {Menu.bit2_1 =0;} // Zellenauswahl aufheben
	    }
  }



/****************************************************************************
 Kalender
 Parameter[in]
 - press:0 = Kalender inkl. Uhrzeit/Datum erstellen
         1 = Btn pressed, aktion ausfhren
		 2 = Uhrzeit/Datum aktualisieren
   X-Pos: 32  57  82  107 132 157 182 207
     +----+---+---+---+---+---+---+---+----+<-  25
	 |    | Mo| Di| Mi| Do| Fr| Sa| So|    | =  41
	 |    +---+---+---+---+---+---+---+    |<-  48
	 |    |   |   |   |   |   |  1|  2|    |
	 |    +---+---+---+---+---+---+---+    |<-  67
	 |    |  3|  4|  5|  6|  7|  8|  9|    | =  81
     |    +---+---+---+---+---+---+---+    |<-  86
	 | <= | 10| 11| 12| 13| 14| 15| 16| => | = 100
	 |    +---+---+---+---+---+---+---+    |<- 105
	 |    | 17| 18| 19| 20| 21| 22| 23|    | = 119
	 |    +---+---+---+---+---+---+---+    |<- 124
	 |    | 24| 25| 26| 27| 28| 29| 30|    | = 138
	 |    +---+---+---+---+---+---+---+    |<- 143
	 |    | 31|   |   |   |   |   |   |    | = 157
	 +----+---+---+---+---+---+---+---+----+<- 162
	 | jedes Jahr |  [Farbe]  |mehrereTage |
	 |            +-----------+            |
	 +------------+  22.Mai   +------------+
	 | 20:15 Uhr  |  31.Mai   |  17.05.10  | 
	 +------------+-----------+------------+<- 222
    1^          82^        158^         237^
 ****************************************************************************/
void kalender(uint8_t press)
 {
   color_set(Hintergrund);
   uint16_t text_color =Menu.Farbe.Text;


	if  (!press)
	    { // Kalender erstellen ...
		  print_Str("Mo"       ,1, 36,41,text_color,no_bg);
          print_Str("Di"       ,1, 61,41,text_color,no_bg);
	      print_Str("Mi"       ,1, 86,41,text_color,no_bg);
	      print_Str("Do"       ,1,111,41,text_color,no_bg);
	      print_Str("Fr"       ,1,136,41,text_color,no_bg);
	      print_Str("Sa"       ,1,161,41,text_color,no_bg);
	      print_Str("So"       ,1,186,41,text_color,no_bg);

          color_set(RAM_Data.Farbe.Button);
          button_build(  2, 25, 31,137,RAM_Data.Farbe.Button); symbole( 1,  10, 91, Menu.Farbe.Text); // Monat Links Btn
          button_build(208, 25,238,137,RAM_Data.Farbe.Button); symbole( 3, 217, 91, Menu.Farbe.Text); // Monat Rechts Btn
          
		  // [jedes Jahr Btn]
		  button_build(  2,162, 77, 30,RAM_Data.Farbe.Button); 
		  if  (Menu.bit1_2) {text_color =Menu.Farbe.TextRot;} else{text_color =Menu.Farbe.Text;} // Merke1=1 (jedes Jahr)
		  print_Str("jedes Jahr", 1,11,174,text_color,no_bg);
          
		  // [Zellenauswahl]
		  if  (Menu.bit2_1) 
		      { // Auswahl fr Start-Tag vorhanden...
			    lcd_Ascii(Menu.Ascii2[0],1, 98,176,Menu.Farbe.Text,no_bg);
				lcd_Ascii(Menu.Ascii2[1],1,104,176,Menu.Farbe.Text,no_bg);
                print_Str("."           ,1,110,176,Menu.Farbe.Text,no_bg);
                lcd_Ascii(Menu.Ascii2[2],1,116,176,Menu.Farbe.Text,no_bg);
				lcd_Ascii(Menu.Ascii2[3],1,122,176,Menu.Farbe.Text,no_bg);
                print_Str(".20"         ,1,128,176,Menu.Farbe.Text,no_bg);
				lcd_Ascii(Menu.Ascii2[4],1,146,176,Menu.Farbe.Text,no_bg);
				lcd_Ascii(Menu.Ascii2[5],1,152,176,Menu.Farbe.Text,no_bg);                
			
			    if  (Menu.bit2_1==2)
		            { // Auswahl fr Ende-Tag vorhanden...
                      print_Str("bis"          ,1,116,188,Menu.Farbe.Text,no_bg);
			          lcd_Ascii(Menu.Ascii2[6] ,1, 98,200,Menu.Farbe.Text,no_bg);
				      lcd_Ascii(Menu.Ascii2[7] ,1,104,200,Menu.Farbe.Text,no_bg);
                      print_Str("."            ,1,110,200,Menu.Farbe.Text,no_bg);
                      lcd_Ascii(Menu.Ascii2[8] ,1,116,200,Menu.Farbe.Text,no_bg);
				      lcd_Ascii(Menu.Ascii2[9] ,1,122,200,Menu.Farbe.Text,no_bg);
                      print_Str(".20"          ,1,128,200,Menu.Farbe.Text,no_bg);
				      lcd_Ascii(Menu.Ascii2[10],1,146,200,Menu.Farbe.Text,no_bg);
				      lcd_Ascii(Menu.Ascii2[11],1,152,200,Menu.Farbe.Text,no_bg);
                    }
              }

          // [Farbauswahl Btn]
		  button_build(159,162, 77, 30,RAM_Data.Farbe.Button);
          color_set(RAM_Data.Farbe.Button);
		  Rectan(169,167, 226, 187, 0, Menu.Farbe.Dunkel +(5 * Menu.Farbe.Sub),1);
        }
  
    if  (!press || press==2)
	    { // Uhrzeit/ Datum erstellen
		  display_time(2);
        }
  
    if  (!press || press==1)
	    { // Kalender Blatt erstellen
          
        }
  }



/*****************************************
 Funktion Eingabe Feld Button (undrckbar)
 Parameter:
 0 Aus        <Feld lschen>
 1 Timer     
 2 Celsius  
 3 RLF
 4 Bodenfeuchte   
 6 Licht    
 7 Kontakt  
 8 Relais
 *****************************************/
void eingabe_fld()
  {
   uint16_t ytop=ytop_eingabe_fld();
   color_set(RAM_Data.Farbe.Button);
  
   Rectan(81, ytop, 158,ytop+30,0,Menu.Farbe.Dunkel+(5*Menu.Farbe.Sub),1); // Eingabe Feld erstellen
  
   if  (!RAM_Data.SenTyp && Menu.Window !=Pumpen_Menu) // Button Feld beschriften
       { // ohne Sensor Nr.
	     print_Str("    Feld    ",1,83,ytop+12,Menu.Farbe.Text,no_bg);
	     print_Str("  l|schen   ",1,86,ytop+24,Menu.Farbe.Text,no_bg);
	   }
   else{
		 switch(Menu.Window)
		       { case Haupt_Menu:
	                  if  (RAM_Data.AnzTyp >2 && RAM_Data.AnzTyp <7)
			              { // Chart/ Liste
				            label_value(0,RAM_Data.SenTyp,RAM_Data.SenNum,1,93,ytop+12,Menu.Farbe.Text,no_bg); // Sensor Label oben
				            mach_dateiname(3, RAM_Data.SenTyp, RAM_Data.SenNum, RAM_Data.AnzTyp);                // Datum unten
                          }
                      else{ label_value(0,RAM_Data.SenTyp,RAM_Data.SenNum,1,93,ytop+18,Menu.Farbe.Text,no_bg);} // nur Sensor Label mitte
                 break;
				 case Relais_Menu:
		              if  (!Menu.bit1_1 && !Menu.bit1_2) // keine aktive Labeleingabe
			              { // Switch Feld Paramter anzeigen
				            switch(Menu.bit2_2)
					              { case 1: print_Str("&",1,87,ytop+13,Menu.Farbe.Text,no_bg); break; // & UND 1.Zeile
			                        case 2: print_Str("/",1,87,ytop+13,Menu.Farbe.Text,no_bg); break; // / ODER 
		                          }
					        label_value(0,RAM_Data.SenTyp,RAM_Data.SenNum,1,81,ytop+13,Menu.Farbe.Text,no_bg); // SensorLabel aus EERPROM ausgeben
					        label_value(5,RAM_Data.SenTyp,RAM_Data.SenNum,1,81,ytop+26,Menu.Farbe.Text,no_bg); // Value aus Puffer Ascii[] ausgeben
				          }
 			          else{ // Aktor oder Sensor Label -eingabe anzeigen
			                label_value(1, RAM_Data.SenTyp,RAM_Data.SenNum,1,93,ytop+19,Menu.Farbe.Text,no_bg);  // Label aus Puffer Ascii[] ausgeben
                          }
                 break;
				 case Pumpen_Menu:
				      if  (Menu.bit2_1)
		                  { // Zelle Auswahl aktiv
				            if  (Menu.bit2_1 ==1)
					            { // Wert Eingabe
						          uint16_t color_foo;
						          for (uint8_t i=0; i<3; i++)
						              { if  (Menu.Cursor >0 && Menu.Cursor-1==i)
							                { color_foo =Menu.Farbe.TextRot;}
                                        else{ color_foo =Menu.Farbe.Text;}
								        print_Zahl(Menu.Ascii[i], 1, 1,111 +i *6, ytop+19, color_foo, no_bg); // Value[0 bis 2] ausgeben
                                      }
			                    }
                            else{ // DosierPumpe beschriften ...
					              label_value(1,RAM_Data.SenTyp,RAM_Data.SenNum,1,93,ytop+19,Menu.Farbe.Text,no_bg);  // Label aus Puffer ausgeben
					            }
                          }
                 break;
               }
       }
  }



/****************************************************************************
 Funktion fr 4 Cursor Tasten zum stellen von Werten
 Parameter[in] flag 0=erstellen 1=Cursor up 2left 3right 4down gedrckt
 ****************************************************************************/
void cursor_btn(uint8_t press)
  {
	color_set(RAM_Data.Farbe.Button);
 
    if  (press)
	    {
		  switch(press) 
		        {
				  case 1: if  (Menu.Value[Menu.Cursor] <9) {Menu.Value[Menu.Cursor] +=1;} else{Menu.Value[Menu.Cursor] =0;} break; // Up
				  case 2: if  (Menu.Cursor >0) {Menu.Cursor -=1;} break; // Left
				  case 3: if  (Menu.Cursor <2) {Menu.Cursor +=1;} break; // Right
				  case 4: if  (Menu.Value[Menu.Cursor] >0) {Menu.Value[Menu.Cursor] -=1;} else{Menu.Value[Menu.Cursor] =9;} break; // Down
                }
          eingabe_fld();
        }
    else{ // Button build
	      button_build(  2,254,77,30,RAM_Data.Farbe.Button); symbole( 1,  36, 263, Menu.Farbe.Text); // <-
	      button_build( 81,221,78,31,RAM_Data.Farbe.Button); symbole( 0, 117, 231, Menu.Farbe.Text); // hoch
          button_build(161,254,77,31,RAM_Data.Farbe.Button); symbole( 3, 195, 263, Menu.Farbe.Text); // ->
          button_build( 81,287,78,31,RAM_Data.Farbe.Button); symbole( 2, 117, 295, Menu.Farbe.Text); // runter
        }
  }



/****************************************************************************
 Funktion Clear Btn lscht;
 im PumpenMen; Zellenwert, DosierWoche
 Parameter[in] flag 0= btn erstellen 1=btn gedrckt, lsch etwas
 ****************************************************************************/
void clear_btn(uint8_t press)
  {
    color_set(RAM_Data.Farbe.Button);

	if  (press)
	    {
		  switch(Menu.Window)
		        { case Pumpen_Menu: if  (Menu.bit2_1) 
				                        { if  (Menu.bit2_1==1) {print_Str("  Wert  ",1,17,299,Menu.Farbe.Text,no_bg);}
	                                      else                 {print_Str("  Name  ",1,17,299,Menu.Farbe.Text,no_bg);}
					                    }
				                    else{ print_Str(" Woche  ",1,17,299,Menu.Farbe.Text,no_bg);}
				                    dosier_tabelle(6);
				  break;
				  case Relais_Menu:   break; //
				  case Kalender_Menu: break; //
			    }
        }
    else{ // Btn build
          button_build( 2,287,78,31,RAM_Data.Farbe.Button);
          print_Str("l|schen",1,20,313,Menu.Farbe.Text,no_bg);
        }
  }



/****************************************************************************
 Funktion Enter Btn speichert;
 im PumpenMen; Zellenwert, DosierProgramm
 Parameter[in] flag 0= btn erstellen 1=btn gedrckt, lsch etwas
 ****************************************************************************/
void enter_btn(uint8_t press)
  {
    color_set(RAM_Data.Farbe.Button);
	
    if  (press)
        {
		  switch(Menu.Window)
	            { case Pumpen_Menu:
		               if  (Menu.bit2_1)
		                   { // Zele ausgewhlt
			                 if  (Menu.bit2_1)
				                 { if  (Menu.bit2_1==1) {print_Str("  Wert  ",1,176,299,Menu.Farbe.Text,no_bg);}
	                               else                 {print_Str("  Name  ",1,176,299,Menu.Farbe.Text,no_bg);}
					             }
				             else{ print_Str(" Woche  ",1,176,299,Menu.Farbe.Text,no_bg);}
				             dosier_tabelle(5); // speichern
                           }
                  break;
                  case Relais_Menu:
		               if  (Menu.bit1_4)
				           { // Schaltfeld Eingabe Aktiv
					         if  (press)
					             {switch_fld(1);}
					       }
                  break;
                }
        }
    else{ // Btn Build
	      button_build( 161,287,77,31,RAM_Data.Farbe.Button);
		  print_Str("speichern",1,173,313,Menu.Farbe.Text,no_bg);
	    }
  }              

