// MMC Funktioen
// AVR interne Module;
#include <avr/pgmspace.h> 
#include <avr/interrupt.h>
#include <stdint.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>       // Watchdog
#include <util/delay.h> // fr MMC notwendig???
#include <stdlib.h> 
#include <string.h> 

#define extern
#include "msr_unit01.h"
#include "buttons.h"
#include "funktionen.h"     // Menu Funktionen
#include "mmc_funktionen.h" // MMC Funktionen
#include "ETT-Display/ett240320tp32revb.h" // ET-TFT Display Funktionen
#include "Clock/clock.h"           // DCF - Uhr Header
#include "SD_Card/fat32.h"         // FAT32 Funktionen
#include "SD_Card/uart_routines.h" // UART Terminal ausgabe Funktionen
#include "SD_Card/spi_routines.h"  // SPI Schnittstelle fr MMC Funktionen
#include "SD_Card/sd_routines.h"   // SpeicherKarten Funktionen
#include "debug.h"                 // abstellbarer Debugmodus fr die Fehlersuche
#undef extern



/****************************************************************************
 Funktion generiert in "fileName[]" einen Datei Namen
 Paramter[in]
 - data_load: 0 Chart speichern, Dateiname aus aktuellem Datum (day, mon, year) erzeugen, 
              1 Chart laden,     Dateiname aus aktuellem Datum und der Men Einstellung(AnzOp1) erzeugen
			  2 Chart lschen,   Dateiname von alten ungltig gewordenen Dateien berechnen
			  3 Datum ausgeben,  Datum einer Datei fr "Charts/Listen" im eingabe_fld ausgeben
			  4 Datum in Menu.Ascii[] speichern 101213

 - sen_typ  ; 3pH 4EC 5C 6rLF 7Bfeuchte 8Fllst 9Licht 10Kont 11Relais 12DosierProgramm
 - sen_num  ; 0-15
 - chart_typ; 3Tag 4Woche 5Monat 6Jahr


 Dateiname;     Beschreibung;
 
 AT120613.CSV | pH1           Tag   12.06.2013
 BW120613.CSV | pH2           Woche 12.06.2013

 CM062013.CSV | EC1           Monat    06.2013
 DJ__2013.CSV | EC2           Jahr        2013

 ET120613.CSV | SHT1          Tag   12.06.2013
 HT120613.CSV | SHT4          Tag   12.06.2013

 IT120613.CSV | Bodenfeuchte1 Tag   12.06.2013
 PT120613.CSV | Bodenfeuchte8 Tag   12.06.2013

 QJ__2013.CSV | Flsltand1    Jahr        2013
 TJ__2013.CSV | Fllstand4    Jahr        2013

 UT010113.CSV = Lichtsensor1  Tag   01.01.2013
 XW010113.CSV = Lichtsensor4  Woche 01.01.2013

 Y00_2012.CSV = Kon 1 "                   2012
 Z01_2012.CSV = Rel 2 "                   2012

 01_DOS_P,CSV = Dosierprogramm 1
 04_DOS_P.CSV = Dosierprogramm 4


mach_dateiname(12, Menu.2bit_1, 0, 0, 0, 0); // Dateiname erzeugen aus mmc_dosierdaten
mach_dateiname(Menu.SenTyp, Menu.SenNum, Menu.AnzTyp, Menu.8bit_1, Menu.8bit_2, Menu.8bit_3); // Datei Name generieren aus ausgabe_fld Btns

mach_dateiname(5, i, 3, day, mon, year); // Datei Name generieren aus Tageschart logdaten spoeichern
mach_dateiname(10, i, 6, day, mon, year % 100); // Datei Name generieren aus kontakte



 ****************************************************************************/
 void mach_dateiname(uint8_t data_load, uint8_t sen_typ, uint8_t sen_num, uint8_t chart_typ)
  {
    uint8_t day_foo  =day;
	uint8_t mon_foo  =mon;
	uint8_t year_foo =year%100;
	uint8_t chars;
    
	uint8_t data_age=0; // Chart speichern
	if  (data_load==1 || data_load==3) {data_age = RAM_Data.AnzOp1;}

    if  (data_load)
	    { // --- Name einer vorhandenen Datei auf MMC erzeugen ---
          switch(RAM_Data.AnzTyp)
	            {
		          case 3: // Tageschart
			              if  (data_load==2) {data_age =8;}
			              if  (day_foo - data_age >0) // datum der datei ... (foo = datei auswahl oder 
				              {day_foo = day_foo - data_age;}
                          else{ 
				                if  (mon_foo==1) // januar?
					                {
						 	          day_foo  = (31 - (data_age - day_foo)); // Tag
							          mon_foo  = 12; // monat
							          year_foo = year_foo -1; // jahr
                                    }
                                else{
						              day_foo = tage_im_monat(mon_foo - 1, year_foo) - (data_age - day_foo);
							          mon_foo = mon_foo -1;
                                    }
                              }
                  break;
    		      case 4: // Wochenchart (datei start immer montag)
			              if  (data_load==2) {data_age =5;}
			              if  (dow-1 < day_foo -(data_age *7)) // datum der datei ermitteln
			            { // fngt die woche im akt. monat an?
				          day_foo =(day_foo - (data_age *7)) - (dow-1);
                        }
                    else{ if  (mon_foo ==1)
				              { // war der monat davor letztes jahr?
				                day_foo  =32 - ((dow-1) -(day_foo -(data_age *7)));
				                mon_foo  =12;
								year_foo =year_foo -1;
                              }
                          else{ // war der monat vorher dieses jahr?
                                day_foo =(tage_im_monat(mon_foo -1, year_foo)+1) -((dow-1) -day_foo);
					            mon_foo =mon_foo -1;
                              }
                        }
                  break;
			      case 5: // Monatschart     mon
			              if  (data_load==2) {data_age =13;}
			              if  (mon_foo -data_age >0) // datum der datei ermitteln
				              {mon_foo = mon_foo -data_age;}
                          else{mon_foo =12 -(data_age -mon_foo); year_foo = year_foo -1;}
                  break;
                  case 6: // Jahreschart
			              if  (data_load==2) {data_age =11;}
			              day_foo =1;
				      	  mon_foo =1;
					      year_foo = year_foo - data_age; // datum der datei ermitteln
                  break;
                }
        
		  // --- Datum im eingabe_fld darstellen --- (nur bei Charts/Listen)
          if  (data_load==3)
              { 
                uint16_t ytop = ytop_eingabe_fld();
                switch(RAM_Data.AnzTyp)
                      {    
			            case 3: print_Zahl(day_foo, 2,1, 96,ytop+24,Menu.Farbe.Text,no_bg); // Tag  10.11.13
   		                        print_Str (".",       1,108,ytop+24,Menu.Farbe.Text,no_bg); // .
   		                        print_Zahl(mon_foo, 2,1,114,ytop+24,Menu.Farbe.Text,no_bg); // Monat
 	                            print_Str (".",       1,126,ytop+24,Menu.Farbe.Text,no_bg); // .
   		                        print_Zahl(year_foo,2,1,132,ytop+24,Menu.Farbe.Text,no_bg); // Jahr
		                break;
				        case 6: print_Zahl(2000 +year_foo,4,1,108,ytop+24,Menu.Farbe.Text,no_bg); // 2013
				        break;
				        default:chars = monat_zu_text(mon_foo); // September
				                uint8_t x_pos =120- (chars *3);
                                for (int8_t i =0; i<chars; i++) {lcd_Ascii(dataString[i],1,x_pos +(i *6),ytop+24,Menu.Farbe.Text,no_bg);}
                        break;
                      }
              }
          else{ if  (data_load==4)
		            { // --- Datei Datum in Menu.Ascii[] ablegen 101213 ---
					  Menu.Ascii[0] = day_foo /10; Menu.Ascii[1] = day_foo %10;
					  Menu.Ascii[2] = mon_foo /10; Menu.Ascii[3] = mon_foo %10;
					  Menu.Ascii[4] = year_foo/10; Menu.Ascii[5] = year_foo%10;
					}
				else{ // --- Dateiname erzeugen ---
		        
                      char var_str[5]; var_str[0] ='\0';
	                  char char_str[26] ="ABCDEFGHIJKLMNOPQRSTUVWXYZ";

                      if  (sen_typ==5 || sen_typ==6)
	                      { fileName[0] = char_str[sen_num+4]; fileName[1] ='\0';} // SHT 0- 3 (E-H)
	                  else{
 	                        switch(sen_typ)
	                              { 
		                            case  3: fileName[0] = char_str[sen_num];    fileName[1] ='\0'; break; // pH           0, 1 (A,B)
			                        case  4: fileName[0] = char_str[sen_num+ 2]; fileName[1] ='\0'; break; // EC           0, 1 (C,D)
	                                case  7: fileName[0] = char_str[sen_num+ 8]; fileName[1] ='\0'; break; // Bodenfeuchte 0- 8 (I-P)
			                        case  8: fileName[0] = char_str[sen_num+16]; fileName[1] ='\0'; break; // Fllstand    0- 4 (Q-T)
			                        case  9: fileName[0] = char_str[sen_num+20]; fileName[1] ='\0'; break; // Lichtsensor  0- 1 (U,X)
                                    default: fileName[0] ='\0';
				                             switch(sen_typ)
						                           {
								                     case 10: strcat((char*)fileName,"Y"); break; // Kontakte
								                     case 11: strcat((char*)fileName,"Z"); break; // Relais
								                   }
                  	                         if  (sen_num <10) {strcat((char*)fileName, "0");} // nur 2 stell. Sensor Nr. // DosierProgramm
	                                         itoa(sen_num, var_str, 10); strcat((char*)fileName, var_str);  // Sensor Nr. // nur die Nummer
                                             strcat((char*)fileName, "_");
						                     if  (sen_typ==12) {strcat((char*)fileName,"Dos_P");}
                                    break;
                                  }
                          }
	                  if  (sen_typ <10)
	                      { // nur bei Chart
		                    switch(chart_typ)
	                              {
	                                case 3: strcat((char*)fileName,"T");    break; // Tageschart
	                                case 4: strcat((char*)fileName,"W");    break; // Wochenchart
	                                case 5: strcat((char*)fileName,"M");    break; // Monatschart
	                                case 6: strcat((char*)fileName, "J__"); break; // Jahreschart
                                  }
                            if  (chart_typ==3)
	                            {
			                      itoa(day_foo, var_str, 10);
			                      if  (day_foo  <10) {strcat((char*)fileName,"0");}
				                  strcat((char*)fileName, var_str); // tag
                                }
                            if  (chart_typ>2 && chart_typ<6)
	                  			{
			           			  itoa(mon_foo, var_str, 10);
			          			  if  (mon_foo  <10) {strcat((char*)fileName,"0");}
				      			  strcat((char*)fileName, var_str); // monat
                    		    }
              		      }
	                  if  (sen_typ <12)
	                      {	  
		                    if  (chart_typ !=3 && chart_typ !=4)
		                        {year_foo = year_foo +2000;} // Jahr 4-stellig
		                    itoa(year_foo, var_str, 10);
		                    strcat((char*)fileName, var_str); // year
                          }
                      // Dateiendung
	                  strcat((char*)fileName, ".CSV");
	
	                  if  (DEBUG)
	                      { 
		                    for (uint8_t i=0; i<12;i++) {transmitByte(fileName[i]);}
     	                    transmitString_F (PSTR(": ")); // Dateiname auf PC-Terminal ausgeben
	                      }
                    }
              }
        }
  }



/********************************************************************************************************************
 CSV Dateiformat;
 Typ:      Time/Date;  Value:     Loginhalt:                  Dateiname;     
 SHT Tag   1232        012,7 056,6 (12:32 hh:mm              ) 1T010113.CSV = SHT 1 Tageschart  01.01.2013 day:mon:year
 SHT Woche 0847        010,4 076,5 (01:08 dow:hh             ) 1W060113.CSV = SHT 1 Wochenchart 06.01.2013 day:mon:year (day =immer montags)
 SHT Monat 3123        018,6 060,5 (31:23 day:hh             ) 1M072012.CSV = SHT 1 Monatschart    07.2012     mon:year
 SHT Jahr  1224        -05,4 065,5 (12:24 mon:day            ) 1J__2012.CSV = SHT 1 Jahreschart       2012         year
 Fllstand 232000 1506 101         (23:20:00 hh:mm:ss day:mon) F01_2013.CSV = Fl 1 Jahreschart       2013         year
 Kontakte  201500 0101 0           (20:15:00 hh:mm:ss day:mon) K01_2012.CSV = Kon 1 "                 2012         year
 Relais    211530 1207 1           (21:15:30 hh:mm:ss day:mon) R01_2012.CSV = Rel 1 "                 2012         year

 ********************************************************************************************************************
 Funktion mmc_log ffnet eine datei oder legt bei write=1 eine Datei an
 falls keine vorhanden ist. dannach wird entweder ein log in die datei
 geschrieben oder eine Datei ausgelesen.
 Parameter[in]
 load:      0= log schreiben
            1= log laden
 sen_typ:   5= Celsius 6=RLF 7=Fllstand 8=Kontakt 9=Relais
 sen_num:   1 bis 14
 chart_typ: 3= tageschart 4=woche 5=monat 6=jahreschart

 Parameter[out]
 return 0; mmc fail
        1; log read/write complete
		2; file not found
 ********************************************************************************************************************/
uint8_t mmc_log(uint8_t load, uint8_t sen_typ, uint8_t sen_num, uint8_t chart_typ)
  {
    color_set(RAM_Data.Farbe.Grund);
	    
    char zahl_str[5];
    int16_t logwert[2];

	int8_t hh_foo =hh;
	int8_t mm_foo =mm;
	int8_t ss_foo =ss;
	int8_t day_foo =day;
	int8_t dow_foo =dow-1;
	int8_t mon_foo =mon;

    // kein MMC Sync? MMC neu initialisieren
    if  (Menu.MMC_Err)
	    { for (int8_t i=0; i<10; i++)
              { Menu.MMC_Err = SD_init();
		        if  (!Menu.MMC_Err) break;
              }
          if  (Menu.MMC_Err)
              { if  (DEBUG)
		            { if  (Menu.MMC_Err == 1) transmitString_F(PSTR("SD card not detected.."));
    	              if  (Menu.MMC_Err == 2) transmitString_F(PSTR("Card Initialization failed.."));
                    }
              }
          else{ Menu.MMC_Err = getBootSectorData (); //read boot sector and keep necessary data in global variables
                if  (Menu.MMC_Err)
                    { if  (DEBUG) {transmitString_F (PSTR("\n\rFAT32 not found!"));}} //FAT32 incompatible drive
              }
          SPI_HIGH_SPEED;//SCK - 4 MHz
          _delay_ms(1); 	//some delay for settling new spi speed
        }

    // Loginhalt schreiben ...
    if  (!load)
	    { dataString[0] ='\0'; // Logzeile wird in dataString aufgenommen
		  zahl_str[0]   ='\0';


	      if  (sen_typ <10)
	          { 
			    if  (sen_typ==5 || sen_typ==6)
				    {
				      logwert[0] = Menu.Value[array_zu_sentyp(2,5,sen_num)]; // Temp
					  logwert[1] = Menu.Value[array_zu_sentyp(2,6,sen_num)]; // rLF
					}
                else{ logwert[0] = Menu.Value[array_zu_sentyp(2,sen_typ,sen_num)];} // pH EC BodenFeuchte Fuellstand Lichtmessung

			     switch(chart_typ)
	                  { // --- pH, EC, C, RLF, BodenF, Fllst, Licht, 1232 012,7 056,6 ---
		                case 3: // Tageschart (hh:mm)
			                    itoa(hh_foo, zahl_str, 10);
								if  (hh_foo <10) {strcat((char*)dataString,"0");}
								strcat((char*)dataString, zahl_str);
					            itoa(mm_foo, zahl_str, 10);
								if  (mm_foo <10) {strcat((char*)dataString,"0");}
								strcat((char*)dataString, zahl_str);
	                    break;
			            case 4: // Wochenchart (dow:hh)
			                    itoa(dow_foo, zahl_str, 10);
								strcat((char*)dataString,"0");
								strcat((char*)dataString, zahl_str);
					            itoa(hh_foo, zahl_str, 10);
								if  (hh_foo <10) {strcat((char*)dataString,"0");}
								strcat((char*)dataString, zahl_str);
		                break;
			            case 5: // Monatschart (day:hh)
			                    itoa(day_foo, zahl_str, 10);
								if  (day_foo <10) {strcat((char*)dataString,"0");}
								strcat((char*)dataString,zahl_str);
					            itoa(hh_foo, zahl_str, 10);
								if  (hh_foo <10) {strcat((char*)dataString,"0");}
								strcat((char*)dataString,zahl_str);
			            break;
			            case 6: // Jahreschart (mon,day)
			                    itoa(mon_foo, zahl_str, 10);
								if  (mon_foo <10) {strcat((char*)dataString,"0");}
								strcat((char*)dataString, zahl_str);
					            itoa(day_foo, zahl_str, 10);
								if  (day_foo <10) {strcat((char*)dataString,"0");}
								strcat((char*)dataString, zahl_str);
		                break;
                      }
              }
          else{ // --- Kontakte und Relais 201500 0101 0 = hh:mm:ss day.mon  ---
		        itoa(hh_foo, zahl_str, 10); if  (hh_foo <10) {strcat((char*)dataString,"0");}; strcat((char*)dataString, zahl_str);
                itoa(mm_foo, zahl_str, 10); if  (mm_foo <10) {strcat((char*)dataString,"0");}; strcat((char*)dataString, zahl_str);
                itoa(ss_foo, zahl_str, 10); if  (ss_foo <10) {strcat((char*)dataString,"0");}; strcat((char*)dataString, zahl_str);
				strcat((char*)dataString, " ");
                itoa(day_foo, zahl_str, 10); if  (day_foo <10) {strcat((char*)dataString,"0");}; strcat((char*)dataString, zahl_str);
				itoa(mon_foo, zahl_str, 10); if  (mon_foo <10) {strcat((char*)dataString,"0");}; strcat((char*)dataString, zahl_str);
              }
          strcat((char*)dataString, " ");

          // Wert Log
          if  (sen_typ==5 && logwert[0] <0) {strcat((char*)dataString, "-"); logwert[0] =logwert[0] -(2 * logwert[0]);}  // SHT Celsius Minuswert
          if  (sen_typ<10) 
	          { // --- pH, EC, Celsius, rLF, BodenF, Fllst, Licht ---
		        for (uint8_t i =0; i<2; i++)
		            {
		   	          if  (logwert[i] / 10 <100) {strcat((char*)dataString, "0");}              // 0
                      if  (logwert[i] / 10 < 10) {strcat((char*)dataString, "0");}              //  0
                      itoa(logwert[i] / 10, zahl_str, 10); strcat((char*)dataString, zahl_str); //   1
          	          strcat((char*)dataString, ",");                                           //    ,
		              itoa(logwert[i] % 10, zahl_str, 10); strcat((char*)dataString, zahl_str); //     9
                      if  (i==0) {strcat((char*)dataString, " ");}                              //
                    }
              }
          else{ // --- Kontakte, Relais ---
		        if  (sen_typ==10) {itoa(Menu.Kontakt[sen_num], zahl_str, 10);} // Kontaktzustand
		        if  (sen_typ==11) {itoa(RAM_Data.Aktor[sen_num].Status, zahl_str, 10);} // Relaiszustand
                if  (sen_typ==12) {itoa(RAM_Data.Aktor[sen_num+16].Status, zahl_str, 10);} // Bewsserungsprogramm
                if  (sen_typ <13) {strcat((char*)dataString, zahl_str);}
              }
          // Zeilenende
          strcat((char*)dataString, "\r\n"); // always end the string with these two characters, before calling the writeFile function
          
		  // Chart/ Liste aktualisieren
          if  (Menu.Window==Haupt_Menu && sen_num==RAM_Data.SenNum && chart_typ==RAM_Data.AnzTyp && RAM_Data.AnzTyp>2 && RAM_Data.AnzTyp<7 && !RAM_Data.AnzOp1)
			  { 
			    if  ((sen_typ==5 && (RAM_Data.SenTyp==5 || RAM_Data.SenTyp==6)) || // sht
				     (sen_typ >2 && sen_typ<10 && sen_typ==RAM_Data.SenTyp)) // pH, EC, BodenF, Fuell, Licht
			        { // Charts ...
					  chart_dot(); // Chart Dot zeichnen
                      chart_minmax(); // Min/Max Wert aktualisieren
                    }
                else{ // Liste fr Kontakte und Relais
				      if  (RAM_Data.SenTyp==sen_typ) {list_entry(1);}
                    }
              }

          // Log auf MMC schreiben
          if  (!writeFile((unsigned char*)fileName))
		      {
			    if  (DEBUG)
				    { for (uint8_t i =0;dataString[i] !='\r'; i++) {transmitByte(dataString[i]);}
					  transmitString_F (PSTR(" data saved \r\n"));
					  Menu.MMC_Err=0; return 1;
					}
              }
        }

   // Log lesen ...
   else{
         uint8_t error = readFile( READ, fileName);
         if  (DEBUG)
             {
			   switch(error)
			         { case 0: transmitString_F(PSTR("File loaded. \n\r")); return 1;
					   case 1: transmitString_F(PSTR("File does not exist.\n\r")); return 2;
			           case 2: transmitString_F(PSTR("File name incompatible.\n\r")); return 2;
                     }
			 }
       }
   return 0; // error
 }



/******************************************************************************
 Funktion list_entry macht einen Logeintrag in der Liste
 "wird von READ_File() FAT32.c aus aufgerufen"
 Parameter:[in] flag 0 =Daten von MMC laden und in Liste eintragen
                     1 =aktuelles Ereigniss in Liste eintragen

 Menu.Auswahl.ChartSeite
 ******************************************************************************/
void list_entry(uint8_t flag)
 { 

    if  (DEBUG)
	    { 
          transmitString_F (PSTR("listentry aus dataString;"));
		  for (int8_t i=0; i<20; i++) {transmitByte(dataString[i]);}
          transmitString_F (PSTR("\n\r"));
        }
   // - zum angeforderten Datum vorspulen...
   // eingelesene datum aus log..foo
   uint8_t tag_foo   =(10 *(dataString[7]-'0'))+(dataString[8] -'0'); // day(01.) aus LogZeile   = 201500 0112 0
   uint8_t monat_foo =(10 *(dataString[9]-'0'))+(dataString[10]-'0'); // mon(12.) aus LogZeile   = 201500 0112 0
   uint8_t jahr_foo  =(10 *(fileName[6]  -'0'))+(fileName[7]   -'0'); // year(13) aus fileName[] ="F01_2013.CSV"
   uint8_t wert_foo  = dataString[12]-'0';
   //  Datum des Charts der angezeigt werden soll ...bar
   uint8_t tag_bar   =(10 * Menu.Ascii[0]) + Menu.Ascii[1];
   uint8_t monat_bar =(10 * Menu.Ascii[2]) + Menu.Ascii[3];
   uint8_t jahr_bar  =(10 * Menu.Ascii[4]) + Menu.Ascii[5];
  /*	Startbedingung beim laden von mmc 
  ListSeiten   =0; Menu.16bit_1  Seiten insg. ergibt sich aus der Anzahl der Logs in der Datei (/22 Eintrge pro Seite)
  ListSeite    =0; RAM...AnzOp2  Seite die gerade angezeigt wird
  ListZelle    =0; Menu.bit6_1   welche Zelle in der Liste kann als nchstes beschrieben werden 0-21
  */
   if  (!flag)
       {
	     if  ((RAM_Data.AnzTyp <5 && (tag_foo <tag_bar || monat_foo <monat_bar || jahr_foo <jahr_bar)) || // Tages/Wochen Chart
              (RAM_Data.AnzTyp==5 && (monat_foo <monat_bar || jahr_foo <jahr_bar)) ||                         // Monats Chart
              (RAM_Data.AnzTyp==6 && jahr_foo <jahr_bar))                                                       // Jahr Chart
	         {return;} // ltere als geforderte Datenlogs berspringen ...
       }
   
   if  ( Menu.bit6_1 >21 || (Menu.bit6_1==21 && wert_foo))
       { // Seite voll
	     Menu.bit16_1 +=1;
		 Menu.bit6_1   =0;
         auswahl_btn(0,16); // Seitenauswahl Btn aktualisieren
		
		 if  (flag)
		     { // bei Ereignissen neue Seite erstellen
			   RAM_Data.AnzOp2 +=1;
		       Rectan(2,25,237,151,0,Menu.Farbe.Grund, 1);
			   list_draw();
			 }
       }
   
   if  (RAM_Data.AnzOp2 == Menu.bit16_1) // Seite = Seitenanzahl
       { // Log/ Ereigniss in die Liste eintragen
         if  (DEBUG) {transmitString_F (PSTR(" schreibe; "));}
 	     color_set(RAM_Data.Farbe.Grund);                      //         Auf (1)     |        Zu (0)
         uint8_t y_foo = ((Menu.bit6_1 /2) *10) +50;           // +----------------------------------------+
         uint8_t x_foo = 0;                                    // |12.01.2013|00:00:12| 12.01.2013|00:00:13| Zelle 0 - 1 
         if  (wert_foo) {x_foo=3;} else{x_foo=122;}            // +----------------------------------------+ * 11 Zeilen
        
		 print_Zahl(tag_foo                                      ,2,1,x_foo    ,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Str("."                                             ,1,x_foo+ 12,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Zahl(monat_foo                                    ,2,1,x_foo+ 18,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Str("."                                             ,1,x_foo+ 30,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Zahl(2000 +jahr_foo,                               4,1,x_foo+ 36,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Zahl(((dataString[0]-'0') *10)+(dataString[1]-'0'),2,1,x_foo+ 66,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Str(":"                                             ,1,x_foo+ 78,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Zahl(((dataString[2]-'0') *10)+(dataString[3]-'0'),2,1,x_foo+ 84,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Str(":"                                             ,1,x_foo+ 96,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
         print_Zahl(((dataString[4]-'0') *10)+(dataString[5]-'0'),2,1,x_foo+102,y_foo,Menu.Farbe.Text,Menu.Farbe.Grund);
	   }
   Menu.bit6_1 +=1; // List Zelle +1
 }



/**************************************************************************************
 Funktion zeichnet Wert als Pixel in ein Chart
 "wird in READ_File() FAT32.c aufgerufen"
 Parameter[in]; flag 0 =Logdaten von MMC als Chart darstellen
                     1 =Log aus einem Mittelwert in den Chart eintragen
 **************************************************************************************/
void chart_dot(void)
  {
    color_set(Hintergrund);
    uint16_t fg_clr  = Menu.Farbe.Text;
	uint16_t bg_clr  = Menu.Farbe.Grund;

    int8_t scale_res=0;   // Skala Auflsung
	int8_t scale_start=0; // Skala Nullpunkt Wert
	int8_t scale_max=0;   // Skala Maximal Wert
	uint8_t x_foo=0;
	char zahl_str[7]; zahl_str[0] ='\0';
	char ch='0';
	int16_t value_foo=0;
	uint8_t value_bar=0;
	int16_t value_baz=0;
    uint8_t negativ_wert =0;
	uint8_t time_foo=0;
	uint8_t time_bar=0;

    // Position (X-Zeitachse) ermitteln ...
    
	// Werte aus dataString entnehmen
    time_foo = (10 * (dataString[0]-'0')) +(dataString[1]-'0');
	time_bar = (10 * (dataString[2]-'0')) +(dataString[3]-'0');

	switch(RAM_Data.SenTyp)
	      {
		    case 2: // Celsius
			        value_foo = (10 * (dataString[6]-'0')) + (dataString[7]-'0');
			        value_bar = dataString[9]-'0';
		            if  (dataString[5] == '-')
				        {negativ_wert = 1;}
				    else{value_foo += ((dataString[5]-'0') *100);}
		    break;
		    case 3: // RLF Wert
		            value_foo = (100 * (dataString[11]-'0')) + (10 * (dataString[12]-'0')) + (dataString[13]-'0');
			        value_bar = dataString[15]-'0';
			break;
		    case 4: // Bodenfeuchte Wert
		            value_foo = (100 * (dataString[12]-'0')) + (10 * (dataString[13]-'0')) + (dataString[14]-'0');
			break;
			case 5: // Lichtmessung Wert
			break;
          }

	switch(RAM_Data.AnzTyp)
		  {
		    case 3: // Tageschart hh mm
    			    if  (time_foo==0 && time_bar==0) // neuer tag/chart?
					    {Menu.bit16_1=-999; Menu.bit16_2=-999; auswahl_btn(0,15); Rectan(45,34,236,134,0,bg_clr,1);}
					x_foo = 45 + (time_foo * 8) + (time_bar / 7); ch ='T';
		    break;
			case 4: // Wochenchart day hh
				    if  (time_foo==0 && time_bar==0)
					    {Menu.bit16_1=-999; Menu.bit16_2=-999; auswahl_btn(0,15); Rectan(55,34,222,134,0,bg_clr,1);} // Fenster lschen
                    x_foo = 55 + (time_foo *24) + time_bar;	ch ='W';
			break;
			case 5: // Monatschart
				    if  (time_foo==0 && time_bar==0)
					    {Menu.bit16_1=-999; Menu.bit16_2=-999; auswahl_btn(0,15); Rectan(45,34,229,134,0,bg_clr,1);}
	                x_foo = 45 + (time_foo * 6) + (time_bar /4); ch ='M';
			break;
			case 6: // Jahreschart
				    if  (time_foo==0 && time_bar==0)
					    {Menu.bit16_1=-999; Menu.bit16_2=-999; auswahl_btn(0,15); Rectan(45,34,237,134,0,bg_clr,1);}
				    x_foo = 45 + (time_foo *16) + (time_bar / 2); ch ='J';
		    break;
          }
	
	// Min Max
	value_baz =(value_foo * 100) + (value_bar * 10);
	if  (negativ_wert) {value_baz = value_baz -(2 * value_baz);}
	if  (Menu.bit16_1==-999) {Menu.bit16_1=value_baz;} else{ if  (value_baz < Menu.bit16_1) {Menu.bit16_1=value_baz;} }
	if  (Menu.bit16_2==-999) {Menu.bit16_2=value_baz;} else{ if  (Menu.bit16_2 < value_baz) {Menu.bit16_2=value_baz;} }

    if  (DEBUG)
	    { 
          transmitString_F (PSTR("Dot aus String;"));
		  for (int8_t i=0; i<20; i++) {transmitByte(dataString[i]);}
          transmitString_F (PSTR("\n\r"));
        }

    // Y-Skala Berechnung
 	switch(RAM_Data.AnzOp2)
	      {
		    case 0: scale_res = 10; break; //  10C insg. 0,1C Schritte
			case 1: scale_res = 20; break; //  20C insg. 0,2C Schritte
			case 2: scale_res = 50; break; //  50C insg. 0,5C Schritte
			case 3: scale_res =100; break; // 100C insg. 1,0C Schritte
          }
    if  (RAM_Data.SenTyp ==5) // Celsius
	    {
		  if  (Menu.bit6_1 <12) // skala hat nur pluswerte
		      { 
		        scale_start = Menu.bit6_1 *10; // startwert auf nullpunkt 0C bis 110C
		        scale_max = scale_start + scale_res; // maxwert der skala
		      }
		  else{ // skala hat auch minuswerte
		        scale_start = ((Menu.bit6_1 -1) % 10) * 10; // startwert
		        if  (scale_start >= scale_res) // skala hat nur minuswerte
		            {scale_max = scale_start - scale_res;} // maxwert ...
                else{scale_max = scale_res - scale_start;} // von minus in pluswerte
		      }
        }
    else{ scale_start = Menu.bit6_1 * 5;
		  scale_max = scale_start + scale_res;
        }

    // Y 34 bis 134
	// bei Negativwerten
  	if  (negativ_wert && Menu.bit6_1 >11 && value_foo <=scale_start && RAM_Data.SenTyp==2) // MinusSkala, Negativwert und Celsius?
	    { 
		  if  (scale_start >= scale_res) // nur negative skala
		      { 
			    if  (value_foo > scale_max) // wert innerhalb des charts?
			        {
					  plot_dot(x_foo, 134 - (((scale_start - value_foo) * (100 / scale_res)) + (value_bar / (scale_res /10))), fg_clr);
				    }
			  }
	  	  else{ // negative und positive skala
			    plot_dot(x_foo, 134 - (((scale_start - value_foo) * (100 / scale_res)) + (value_bar / (scale_res /10))), fg_clr);
			  }
		}
	// bei Pluswerten
    if  (negativ_wert==0)
	    { // skala ohne minusbereich
		  if  (Menu.bit6_1 <12 && value_foo >= scale_start && value_foo < scale_max)
		      { // ist der wert innerhalb der skalierung?
		        plot_dot( x_foo, 134 - ((value_foo - scale_start) * (100 / scale_res)) - (value_bar / (scale_res /10)), fg_clr);
              }
          // skala mit minusbereich
          if  (Menu.bit6_1 >11 && scale_res > scale_start && value_foo < scale_max)
			  {
			    plot_dot( x_foo, 134 - ((scale_start * (100 / scale_res)) + value_foo + (value_bar / (scale_res / 10))), fg_clr);
              }
		}
  }



/******************************************************************************
 Funktion mmc_dosierdaten speichert Menu.Tabellenwoche... auf MMC  oder ladet
 von MMC in Menu.Tabellenwoche...
 Parameter[in]: flag 0=lesen 1=schreiben
 Parameter[out]: return 0 =fail
                 return 1 =ok
 ******************************************************************************/
uint8_t mmc_dosierdaten(uint8_t write)
  {
    // FileName des Dosierprogramms generieren
	char zahl_str[3];
    zahl_str[0] ='\0';
    fileName[0] ='\0';
    
    // Daten schreiben?
	if  (write)
	    {
		  mach_dateiname(0, 12, Menu.bit2_2, 0); // Dateiname erzeugen
	      deleteFile (fileName);                       // alte Datei lschen falls vorhanden

		  uint8_t zeile_leer=1;
		  for (uint8_t woche=0; woche<16; woche++)
		      { 
				for (uint8_t zeile=0; zeile<9; zeile++) // 9 Zeilen
		            {
					  for (uint8_t spalte=0; spalte<7; spalte++) // 7 Spalten
				          { if  (Menu.Tabellenwoche[woche].Zeile[zeile].Spalte[spalte])
					            { // Zellenwert in dataString aufnehmen...
								  if  (zeile_leer)
								      {
								        itoa(woche,  zahl_str, 10);
								        if (woche<10) {strcat((char*)dataString, "0");}
								        strcat((char*)dataString, zahl_str); // Woche 00 bis 15
 		                                strcat((char*)dataString, " ");
						                zeile_leer =0;
                                      }
								  itoa(zeile,  zahl_str, 10); strcat((char*)dataString, zahl_str);         // Zeile 0 bis 8
							      itoa(spalte, zahl_str, 10); strcat((char*)dataString, zahl_str);         // Spalte 0 bis 6
								  itoa(Menu.Tabellenwoche[woche].Zeile[zeile].Spalte[spalte], zahl_str, 10); 
								  if  (Menu.Tabellenwoche[woche].Zeile[zeile].Spalte[spalte] <100) {strcat((char*)dataString, "0");}
								  if  (Menu.Tabellenwoche[woche].Zeile[zeile].Spalte[spalte] < 10) {strcat((char*)dataString, "0");}
								  strcat((char*)dataString, zahl_str); // Zellenwert
							      strcat((char*)dataString," ");
                                }
                          }
                      if  (!zeile_leer)
					      { // Zeile in Datei schreiben
						    zeile_leer=1;
						    strcat((char*)dataString, "\r\n");
							mach_dateiname(0, 12, Menu.bit2_2, 0);  // Dateiname erzeugen
							if  (writeFile((unsigned char*)fileName))
							    {if  (DEBUG) {transmitString_F(PSTR("File write error!\n\r"));}}	// Datei schreiben
							dataString[0]='\0'; // dataString lschen
					      }
		            }
			  }
        }

    // Daten lesen?
    else{ // alle Dosierdaten zum Programm von MMC in Menu.Dosiertabelle[]... laden
          mach_dateiname(0,12,Menu.bit2_2,0);  // Dateiname erzeugen
		  uint8_t error = readFile( READ, fileName);
          if  (DEBUG)
              {
			    if  (error==0)
				    { 
					  if  (DEBUG) {transmitString_F(PSTR("File loaded.\n\r"));}
				    }
				else{
				      if  (DEBUG) {transmitString_F(PSTR("File not found.\n\r"));}
				      return 0;
				    }
			  }
	    }
    return 1;
  }


/******************************************************************************
 Funktion schreibt die Dosierdaten einer Zeile in die Menustruktur.
 ******************************************************************************/
 void dos_zeile(void)
  {
    transmitString_F (PSTR("dos_zeile aufruf()\n\r"));
    uint8_t woche_foo  = (10 * (dataString[0]-'0')) +(dataString[1]-'0');
	uint8_t spalte_foo;
	uint8_t zeile_foo;

	for (uint8_t pos=3; dataString[pos] !='\r'; pos+=6)
	    {
		  zeile_foo  = dataString[pos]-'0';
          spalte_foo = dataString[pos+1]-'0';

		  Menu.Tabellenwoche[woche_foo].Zeile[zeile_foo].Spalte[spalte_foo] = (100 * (dataString[pos+2]-'0')) +(10 * (dataString[pos+3]-'0')) +(dataString[pos+4]-'0');
          transmitString_F (PSTR("woche:")); 
	      transmitByte(dataString[0]); 
		  transmitByte(dataString[1]);
		  transmitString_F (PSTR(" "));
          transmitString_F (PSTR("zeile:")); transmitByte(dataString[pos]); transmitString_F (PSTR(" "));
		  transmitString_F (PSTR("spalte:")); transmitByte(dataString[pos+1]); transmitString_F (PSTR(" "));
		  transmitByte(dataString[pos+2]);
          transmitByte(dataString[pos+3]);
          transmitByte(dataString[pos+4]);
		  transmitString_F (PSTR("\n\r"));
        }
    dataString[0]='\0'; // puffer lschen
  }


