/*######################################################################################
AVR Main-Header



#######################################################################################*/


#ifndef _MSR_UNIT01_H_
#define _MSR_UNIT01_H_



// interne Header
// MSR_Automat Header

#include <stdint.h>
#include <avr/eeprom.h>

void update_jede_sek(void);
void update_10sek(void);



#define Kal_Funktion  0
#define Kal_Menu      1
#define Haupt_Menu    2 
#define Relais_Menu   3
#define Relais_Liste  4
#define Info_Menu     5
#define Kalender_Menu 6
#define Pumpen_Menu   7


#define left  1
#define up    2
#define down  3
#define right 4

unsigned char fileName[13]; // MMC Dateiname Puffer
char monat_txt[10];         // Monat als Text char-array




/*------------------------------ =RAM= flchtige Variablen -----------------------------+
 |                                                                                      |
 |                                  flashRAM    1102Bytes                               |
 +--------------------------------------------------------------------------------------*/
typedef struct
  { // --- (100Bytes) ---
    uint8_t Ascii[14]; // zur Label oder Value Ausgabe
	uint8_t Ascii2[14];// Monat als Text oder 2x Datum Kalenderauswahl [140113150113]
	int16_t bit16_1  ; // Chart =MinWert, Listen =SeitenAnzahl
	int16_t bit16_2  ; // Chart =MaxWert

    // 0,1=pH  2,3=EC  4-7=SHTtemp 8-11=SHTfeucht  12-19=BodenF  20-23=Fllst  24-27=Licht
    int16_t Value[28];  // aktuelle Sensor Messwerte, -32000 oder kleiner = SensorFail
    uint8_t Kontakt[8]; // Kontakt Zustand

    // --- Men & System Variablen (7 Bytes) ---
    struct{ uint16_t SHT_Nr  : 2, // Welcher SHT Sensor soll ausgelesen werden? 0-3
			         DCF_On  : 1, // 1= Uhr nach DCF stellen wenn signal vorhanden
					 Bclocked: 1, // Analog oder Digital Uhr oder Uhr/Datum im Kalendermen aktualisiert (Anzeige clocked)
                     Sclocked: 1, // kleiner Uhr Btn im Hauptmen aktualisiert (kleine Uhr in der Leiste clocked) 0=nicht 1=aktuell
					 Clocked : 1, // 1= System Uhr wurde gestellt
					 Cursor  : 4, // Cursor anzeigen? 0 nein, 1-9 = Cursor Position bei eingabe.  Cursor 1= "_abel-001" Cursor 9= "Label_00_"
                     GrKl    : 2,
			         Minus   : 1,
                     UndOder : 1,
					 MMC_Err : 2; // 0=Card OK, 1=SD card not detected oder FAT32 incompatible drive, 2 =Card Initialization failed
            uint16_t Tab     : 7, // Tab = Aktuelle Buttonauswahl Position fr Btn Rahmen und zur Unterscheidung (0 bis 127)
					 TabTimer: 3, // Timer (max.31s) bis der Tab(Btn Auswahlrahmen) zurckgestellt wird //*** 30 bits bis hier***
					 Window  : 4, // 0=Kalib 1=KalMen 2=HMen 3=RelMen 4=RelMen2 5=InfoMen 6=UhrMen 7=EinstellungsMen
			         bit1_1  : 1, // 1Bit Merker 1 (0 oder 1)
                     bit1_2  : 1; // 1Bit Merker 2 (")
		    uint16_t bit1_3  : 1, // 1Bit Merker 3 (")
			         bit1_4  : 1, // 1Bit Merker 4 (")
					 bit2_1  : 2, // 2Bit Merker 1 (0 bis  3)
					 bit2_2  : 2, // 2Bit Merker 2 (")
					 bit2_3  : 2, // 2Bit Merker 3 (")
			         bit4_1  : 4, // 4Bit Merker 1 (0 bis 15)
                     bit4_2  : 4; // 4Bit Merker 2 (")
		    uint16_t bit4_3  : 4, // 4Bit Merker 3 (")
                     bit6_1  : 6, // 6Bit Merker 1 (0 bis 63)
					 frei    : 6;
          };

    // --- Men Screen & Button Farben (14 Bytes) ---
    struct{ uint16_t Grund      ; // Backgroundfarbe; Schwarz oder Weiss
            uint16_t Text       ; // Textfarbe; Schwarz fr helle, Weiss fr dunkle Buttons
            uint16_t TextRot    ; // Textfarbe; Dunkelrot fr helle, Hellrot fr dunkle Buttons
            uint16_t TextGruen  ; // Textfarbe; Dunkelgrn fr helle, Hellgrn fr dunkle Buttons
            uint16_t Hell       ; // Hellste Farbe des mehrfarbigen Buttons
            uint16_t Dunkel     ; // Dunkelste Farbe des mehrfarbigen Buttons
            uint16_t Sub        ; // Farbabzug beim ColorRange
          } Farbe; // werden mit color_set() zur laufzeit eingestellt.

    // --- Dosier Tabellen Puffer (1008 Bytes) ---
    struct{ struct{ uint8_t Spalte[7]; // [0]Montag [1]Dienstag ... nur Werte von 0 bis 255
                  } Zeile[9];
          } Tabellenwoche[16];
  } Menu_struct ; Menu_struct Menu;




/*---------------------------------- =EEPROM= nichtflchtige Variablen -------------------------------+
 |							   	                      4K Bytes insgesamt					          |
 | +------------+ +-----------+ +-----------+ +------------+ +------------+ +----------+ +----------+ |
 | |Kalibrierung| |Men Farben| |Menu.Einst.| |SenAktLabel | | Aktor Eins.| |Alarm-Eins| | frei ca. | |
 | |  29 Bytes  | | 11 Bytes  | | 33 Bytes  | |  621 Bytes | | 932 Bytes  | | ??? Bytes| |2374Bytes | |
 | +------------+ +-----------+ +-----------+ +------------+ +------------+ +----------+ +----------+ |
 +----------------------------------------------------------------------------------------------------*/

typedef struct
  { 
	// --- Touch Screen Kalibrierung (29 Byte) ---
    struct{ int32_t Divider		  ;
            int32_t An     		  ;
            int32_t Bn     		  ;
            int32_t Cn     		  ;
            int32_t Dn     		  ;
            int32_t En            ;
            int32_t Fn            ;
			uint8_t Saved         ; // 255=nein, 1=ja
          } TouchScreenCalibrating;
    // --- Menu Farben (11 Bytes) ---
    struct{ uint8_t  Zeile[9];
	        uint16_t Leiste : 5, // Menleisten Farbe
	                 Button : 5, // Menbutton Farbe
				     Grund  : 1, // Men Hintergrund Farbe(Schwarz oder Weiss)
					 frei   : 4; 
          } Farbe;
    // --- HauptMen Einstellungen (33 Bytes) ---
    struct{ // --- byte wird nach erststart mit 0x00 beschrieben ---
		    uint16_t SenTyp : 4, // Hauptmen Anzeige Einstellung
			         SenNum : 4, // "
					 AnzTyp : 4, // "
					 AnzOp1 : 4; // "
            uint16_t AnzOp2    ; // "
			uint8_t  AnzOp3 : 4, // "
			         DCF_On : 1, // 0=OFF  1=ON
					 RTC_Run: 1, // 0=Nein 1=RTC (DS1307) luft ber Batterie und hat Uhrzeit/Datum
					 frei   : 1;
          };
    struct{ uint16_t SenTyp: 4, // 0 bis 15
                     SenNum: 4, // 0 bis 15
                     AnzOp1: 4, // 0 bis 15  
	                 AnzOp2: 4; // 0 bis 15
          } Feld[14]; //  0-8 = 9DigF, 9-12 = 4DigF, 13=1Digital Feld

    // ---- Sensor,-Aktor,-Label (621 Bytes) ---
    
    struct{ char Ascii[9];
	      } Label[69]; // (0,1=pH 2,3=EC 4-11=SHT 12-19=BodenF 20-23=Fllst 24-27=Licht 28-35=Kontakte 36-51=Relais 52-55=DosProgr. 56-64=DosPump. 65-68=Magn.Ventile

    // --- Aktor Schaltbedingungen (932 Bytes) ---
    struct { // Aktoren sind;     Relais  Bew.Programm  Alarm
	         uint8_t Modus :2, // Modus   Modus         Modus   0=Off,1=On,2=auto nach Schaltbedingungen
			         Status:1, // Status  Status        Status  0=Off,1=On
					 Option:5; // PDelay  PumpeNr       Meldung 0-31 =1-30 Minuten PowerDelay, 0-3 MagnV, 0-? welche Meldungen sollen gemacht werden?
            
			 struct{ uint8_t  Value[4];   // 6 Zeichen fr Schaltwert
			         uint16_t SenTyp  :4, // Sensor Typ (0-15)
  	                          SenNum  :5, // Sensor Nr. (0-31)
                              Grk     :2, // 0 ohne, 1=< , 2=>
		                      Neg     :1, // 0 positiver Wert, 1 negativer Wert
			                  Aor     :2; // 0=AND, 1=OR 2=ohne
                   } Schaltbedingung[6];  // 6 mal pro Relais 0 1 2 =Ein 3 4 5 =Auschaltbedingung
           } Aktor[24]; // 0-15 =16Relais  16-19 =4B-Programme  20-23 =Alarme(Akustisch/ SMS/ Blitzleuchte/ LED?)

  } SaveData;
extern SaveData RAM_Data; // RAM_Data Struktur im RAM
extern SaveData EEP_Data; // EEP_Data Struktur im EEPROM


#endif


