/* Update Funktionen werden jede Sekunde aufgerufen */

// AVR-Studio interne Module
#include <string.h>
#include <stdint.h>
#include <avr/wdt.h>       // Watchdog

// Projekt Module;
#define extern
#include "update.h"
#include "msr_unit01.h"
#include "buttons.h"
#include "funktionen.h"
#include "mmc_funktionen.h"
#include "Kontakte/kontakte.h"
#include "sht/sht11.h"
#include "Clock/clock.h" // DCF - Uhr Header
#undef extern



/********************************************************************
 wird jede Sekunde ausfhrt
 ********************************************************************/
void update_jede_sek()
  { 
    wdt_reset();
   
   	if  (ss % 10 ==0) {update_10sek();} // alle 10sek. ausfhren

    if  (!Menu.Clocked && RAM_Data.DCF_On && flags.dcf_sync) // Uhr nicht gestellt, DCF ON und ein komplettes DCF-Signal erfolgreich ausgewertet?
	    {
		  Menu.Clocked       =1; // hh:mm:ss day.mon.year dow sind jetzt aktuell
		  update.small_clock =1; // kleinen UhrBtn aktualisieren
		  update.big_clock   =1; // groe DAUhr oder Uhr/Datum im KalenderMen ktualisieren
        }
	/*if  (Menu.Sekunde % 2 ==0 && Menu.Window !=Kal_Funktion)
	    {tab_blink(1);} else {tab_blink(0);}; // Btn Auswahlrahmen blinken */ //berarbeiten  hier***

	switch(Menu.Window)
	      { case Haupt_Menu:
			     uhr_fld(1); // Uhrfeld ...
		         info_fld(1); // Infofeld ...
		         ausgabe_fld(2); // Displayfeld aktualisieren
            break;
			case Kalender_Menu:
			     kalender(2); // Uhrzeit/ Datum aktualisieren
			break;
	      }

    // 4 SHT Sensoren
    if  (sht11_measure_finish() ) // Sht Mess-Intervall fr 4 Sensoren
        {
   	      Menu.Value[Menu.SHT_Nr +4] = sht11_get_tmp();    // Temp holen
		  Menu.Value[Menu.SHT_Nr +8] = sht11_get_hum(); // Feuchte holen

		  if  (Menu.SHT_Nr < 3)
		      {Menu.SHT_Nr ++;}
	      else{Menu.SHT_Nr =0;} // - nchster SHT Sensor (0-3)

		  sht11_start_measure(Menu.SHT_Nr); // - Messsung starten
        }
  };


/**************************************************************************************
 Funktion die einmal alle 10sek aufgerufen wird
 **************************************************************************************/
void update_10sek()
  {
    // Btn AuswahlRahmen Timer bis zum zurcksetzen
	if  (!Menu.Tab==1 && Menu.TabTimer >0)
		{Menu.TabTimer--;} 
	else{Menu.TabTimer =4; tab_blink(0); Menu.Tab=1;} // Tab nach 40 Sek. zurck auf Menleiste
   
	if  (Menu.Window ==Haupt_Menu)
	    { 
          if  (RAM_Data.AnzTyp >2 && RAM_Data.AnzTyp <7 && RAM_Data.SenTyp>2 && RAM_Data.SenTyp <10)
		      {
			  //uint8_t 
			    int16_t Val = Menu.Value[array_zu_sentyp(2,RAM_Data.SenTyp,RAM_Data.SenNum)];
				// 0,1=pH  2,3=EC  4-7=SHTtemp 8-11=SHTfeucht  12-19=BodenF  20-23=Fllst  24-27=Licht
				if  (Val > -32000) // wenn ein gltiger SensorWert vorhanden ist
				    { // Min/ Max fr einen eingestellten Chart berechnen
					  if  (Menu.bit16_1 > Val)
					      { // neuen MinWert bernehmen
						    Menu.bit16_1 = Val;
                          }
                      if  (Menu.bit16_2 < Val)
					      { // neuen MaxWert bernehmen
						    Menu.bit16_2 = Val;
					      }
				    }
			  }
        }
  
	// immer nach 7min 30sek (Sensor_Log speichern)
	if  (Menu.Clocked && ((ss== 0 && (mm==0 || mm==15 || mm==30 || mm==45)) || 
	                      (ss==30 && (mm==7 || mm==22 || mm==37 || mm==52))) )
		{
          for (uint8_t i = 0; i < 28; i++)
		      {
			    if  (Menu.Value[i] > -32000) // gltiger Sensorwert vorhanden?
			        { 
					  uint8_t sen_typ =array_zu_sentyp(0,0,i);
					  uint8_t sen_num =array_zu_sentyp(1,0,i);
                      if  (sen_typ ==6) {sen_typ =5;} // SHT Temp & Feuchte in einer Logdatei

					   // 2.stelle hinterm komma lschen
					  if  (Menu.Value[i] % 10 >4)
					      {Menu.Value[i] = Menu.Value[i] + (10 -(Menu.Value[i] % 10));}
                      Menu.Value[i] = Menu.Value[i] / 10;
                      
					  // Tageschart
					  mach_dateiname(0, sen_typ, sen_num, 3); // Datei Name zum speichern eines TagesCharts(3) erzeugen
					  if  (!mmc_log(0, sen_typ, sen_num, 3)) {Menu.MMC_Err =1;} // Log speichern

					  // Wochenchart
                      if  (mm==0) // jede Stunde
		                  { 
                            mach_dateiname(0, sen_typ, sen_num, 4); // Datei Name zum speichern des Wochenchart erzeugen
						    if  (!mmc_log(0, sen_typ, sen_num, 4)) {Menu.MMC_Err =1;} // speichern

							// Monatschart
						    if  (hh % 4==0) // alle vier Stunden
                                {
                                  mach_dateiname(0, sen_typ, sen_num, 5); // Datei Name zum speichern eines Monatscharts erzeugen
						    	  if  (!mmc_log(0, sen_typ, sen_num, 5)) {Menu.MMC_Err =1;} // speichern

						          // Jahreschart
				                  if  (hh==0)
							          {
								        if  (day % 2==0) // alle zwei tage ...
                                            {
                                              mach_dateiname(0, sen_typ, sen_num, 6); // Datei Name zum speichern eines Jahrescharts erzeugen
					                          if  (!mmc_log(0, sen_typ, sen_num, 6)) {Menu.MMC_Err =1;}// speichern
                                            }
                                      }
                               }
                          }
                    }
			  }
        }
  }



/*******************************************************
 Kontakt nderung
 *******************************************************/
void kontakt_update()
  { // 8 Kontakte
    uint8_t bit=0;
    for (uint8_t i=0; i<7;i++)
        {
          if  ( !(PINK & (1 << (i))))
	          {bit =0;} // Zu
          else{bit =1;} // Offen
	      if  (Menu.Kontakt[i] !=bit)
	          { // Vergleichswerte aktualisieren
		        Menu.Kontakt[i] =bit;
		  	    if  (Menu.Clocked && !Menu.MMC_Err)
                    {
					  mach_dateiname(0, 10, i, 6); // Datei Name zum speichern einer Kontaktmeldung erzeugen
				      // nderung auf MMC speichern
				      if  (1==mmc_log(0,10, i, 0))
					      {Menu.MMC_Err =0;} // OK
                      else{Menu.MMC_Err =1;} // Fail
                    }		
		      }
        }
    Kontakte_Port = PINK;
  };



